@@ -1,5 +1,5 @@
 /*
- * $Id: downloads.c 7393 2009-04-22 23:10:15Z dothebart $
+ * $Id: downloads.c 8065 2009-11-19 19:52:39Z dothebart $
  */
 #include "webcit.h"
 #include "webserver.h"
@@ -175,7 +175,7 @@ HashList* LoadFileList(StrBuf *Target, WCTemplputParams *TP)
 	int HavePic;
 	WCTemplputParams SubTP;
 
-	memset(&TP, 0, sizeof(WCTemplputParams));
+	memset(&SubTP, 0, sizeof(WCTemplputParams));
 	serv_puts("RDIR");
 	serv_getln(buf, sizeof buf);
 	if (buf[0] != '1') return NULL;
@@ -240,46 +240,47 @@ void display_mime_icon(void)
 
 void download_file(void)
 {
-	char buf[256];
+	wcsession *WCC = WC;
+	StrBuf *Buf;
 	off_t bytes;
-	char content_type[256];
-	char *content = NULL;
+	StrBuf *ContentType = NewStrBufPlain(HKEY("application/octet-stream"));
 
 	/* Setting to nonzero forces a MIME type of application/octet-stream */
 	int force_download = 1;
 	
-	safestrncpy(buf, ChrPtr(WC->UrlFragment2), sizeof buf);
-	unescape_input(buf);
-	serv_printf("OPEN %s", buf);
-	serv_getln(buf, sizeof buf);
-	if (buf[0] == '2') {
-		bytes = extract_long(&buf[4], 0);
-		content = malloc(bytes + 2);
-		if (force_download) {
-			strcpy(content_type, "application/octet-stream");
+	Buf = NewStrBuf();
+	StrBufExtract_token(Buf, WCC->Hdr->HR.ReqLine, 0, '/');
+	StrBufUnescape(Buf, 1);
+	serv_printf("OPEN %s", ChrPtr(Buf));
+	StrBuf_ServGetln(Buf);
+	if (GetServerStatus(Buf, NULL) == 2) {
+		StrBufCutLeft(Buf, 4);
+		bytes = StrBufExtract_long(Buf, 0, '|');
+		if (!force_download) {
+			StrBufExtract_token(ContentType, Buf, 3, '|');
 		}
-		else {
-			extract_token(content_type, &buf[4], 3, '|', sizeof content_type);
-		}
-		output_headers(0, 0, 0, 0, 0, 0);
-		read_server_binary(WC->WBuf, bytes);
+		read_server_binary(WCC->WBuf, bytes, Buf);
 		serv_puts("CLOS");
-		serv_getln(buf, sizeof buf);
-		http_transmit_thing(content_type, 0);
-		free(content);
+		StrBuf_ServGetln(Buf);
+		http_transmit_thing(ChrPtr(ContentType), 0);
 	} else {
-		hprintf("HTTP/1.1 404 %s\n", &buf[4]);
+		StrBufCutLeft(Buf, 4);
+		hprintf("HTTP/1.1 404 %s\n", ChrPtr(Buf));
 		output_headers(0, 0, 0, 0, 0, 0);
 		hprintf("Content-Type: text/plain\r\n");
-		wprintf(_("An error occurred while retrieving this file: %s\n"), &buf[4]);
+		wc_printf(_("An error occurred while retrieving this file: %s\n"), 
+			ChrPtr(Buf));
 		end_burst();
 	}
+	FreeStrBuf(&ContentType);
+	FreeStrBuf(&Buf);
 }
 
 
 
 void delete_file(void)
 {
+	const StrBuf *MimeType;
 	StrBuf *Buf;
 	char buf[256];
 	
@@ -291,9 +292,8 @@ void delete_file(void)
 	GetServerStatus(Buf, NULL);
 	StrBufCutLeft(Buf, 4);
 	strcpy(WC->ImportantMessage, ChrPtr(Buf));
-	do_template("files", CTX_NONE);
-	output_headers(0, 0, 0, 0, 0, 0);
-	end_burst();
+	MimeType = DoTemplate(HKEY("files"), NULL, &NoCtx);
+	http_transmit_thing(ChrPtr(MimeType), 0);
 	FreeStrBuf(&Buf);
 }
 
@@ -301,21 +301,21 @@ void delete_file(void)
 
 void upload_file(void)
 {
+	const StrBuf *RetMimeType;
 	const char *MimeType;
 	char buf[1024];
 	long bytes_transmitted = 0;
 	long blocksize;
 	wcsession *WCC = WC;     /* stack this for faster access (WC is a function) */
 
-	MimeType = GuessMimeType(WCC->upload, WCC->upload_length); 
+	MimeType = GuessMimeType(ChrPtr(WCC->upload), WCC->upload_length); 
 	serv_printf("UOPN %s|%s|%s", WCC->upload_filename, MimeType, bstr("description"));
 	serv_getln(buf, sizeof buf);
 	if (buf[0] != '2')
 	{
 		strcpy(WCC->ImportantMessage, &buf[4]);
-		do_template("files", NULL);
-		output_headers(0, 0, 0, 0, 0, 0);
-		end_burst();
+		RetMimeType = DoTemplate(HKEY("files"), NULL, &NoCtx);
+		http_transmit_thing(ChrPtr(RetMimeType), 0);
 		return;
 	}
 
@@ -331,7 +331,7 @@ void upload_file(void)
 		if (buf[0] == '7')
 		{
 			blocksize = atoi(&buf[4]);
-			serv_write(&WCC->upload[bytes_transmitted], blocksize);
+			serv_write(&ChrPtr(WCC->upload)[bytes_transmitted], blocksize);
 			bytes_transmitted += blocksize;
 		}
 	}
@@ -339,9 +339,8 @@ void upload_file(void)
 	serv_puts("UCLS 1");
 	serv_getln(buf, sizeof buf);
 	strcpy(WCC->ImportantMessage, &buf[4]);
-	do_template("files", CTX_NONE);
-	output_headers(0, 0, 0, 0, 0, 0);
-	end_burst();
+	RetMimeType = DoTemplate(HKEY("files"), NULL, &NoCtx);
+	http_transmit_thing(ChrPtr(RetMimeType), 0);
 }
 
 
@@ -352,28 +351,30 @@ void upload_file(void)
  */
 void output_image(void)
 {
-	char blank_gif[SIZ];
+	StrBuf *Buf;
 	wcsession *WCC = WC;
-	char buf[SIZ];
 	off_t bytes;
 	const char *MimeType;
 	
+	Buf = NewStrBuf();
 	serv_printf("OIMG %s|%s", bstr("name"), bstr("parm"));
-	serv_getln(buf, sizeof buf);
-	if (buf[0] == '2') {
-		bytes = extract_long(&buf[4], 0);
+	StrBuf_ServGetln(Buf);
+	if (GetServerStatus(Buf, NULL) == 2) {
+		StrBufCutLeft(Buf, 4);
+		bytes = StrBufExtract_long(Buf, 0, '|');
 
 		/** Read it from the server */
 		
-		if (read_server_binary(WCC->WBuf, bytes) > 0) {
+		if (read_server_binary(WCC->WBuf, bytes, Buf) > 0) {
 			serv_puts("CLOS");
-			serv_getln(buf, sizeof buf);
+			StrBuf_ServGetln(Buf);
 		
 			MimeType = GuessMimeType (ChrPtr(WCC->WBuf), StrLength(WCC->WBuf));
 			/** Write it to the browser */
 			if (!IsEmptyStr(MimeType))
 			{
 				http_transmit_thing(MimeType, 0);
+				FreeStrBuf(&Buf);
 				return;
 			}
 		}
@@ -385,8 +386,9 @@ void output_image(void)
 	 * Instead of an ugly 404, send a 1x1 transparent GIF
 	 * when there's no such image on the server.
 	 */
-	snprintf (blank_gif, SIZ, "%s%s", static_dirs[0], "/blank.gif");
-	output_static(blank_gif);
+	StrBufPrintf (Buf, "%s%s", static_dirs[0], "/blank.gif");
+	output_static(ChrPtr(Buf));
+	FreeStrBuf(&Buf);
 }
 
 void 
@@ -429,16 +431,16 @@ InitModule_DOWNLOAD
 			 GroupchangeFilelistBySequence,
 			 CTX_FILELIST);
 
-	RegisterNamespace("FILE:NAME", 0, 2, tmplput_FILE_NAME, CTX_FILELIST);
-	RegisterNamespace("FILE:SIZE", 0, 1, tmplput_FILE_SIZE, CTX_FILELIST);
-	RegisterNamespace("FILE:MIMETYPE", 0, 2, tmplput_FILEMIMETYPE, CTX_FILELIST);
-	RegisterNamespace("FILE:COMMENT", 0, 2, tmplput_FILE_COMMENT, CTX_FILELIST);
+	RegisterNamespace("FILE:NAME", 0, 2, tmplput_FILE_NAME, NULL, CTX_FILELIST);
+	RegisterNamespace("FILE:SIZE", 0, 1, tmplput_FILE_SIZE, NULL, CTX_FILELIST);
+	RegisterNamespace("FILE:MIMETYPE", 0, 2, tmplput_FILEMIMETYPE, NULL, CTX_FILELIST);
+	RegisterNamespace("FILE:COMMENT", 0, 2, tmplput_FILE_COMMENT, NULL, CTX_FILELIST);
 
 	RegisterConditional(HKEY("COND:FILE:ISPIC"), 0, Conditional_FILE_ISPIC, CTX_FILELIST);
 
-	WebcitAddUrlHandler(HKEY("image"), output_image, ANONYMOUS);
-	WebcitAddUrlHandler(HKEY("display_mime_icon"), display_mime_icon , ANONYMOUS);
-	WebcitAddUrlHandler(HKEY("download_file"), download_file, NEED_URL);
-	WebcitAddUrlHandler(HKEY("delete_file"), delete_file, NEED_URL);
-	WebcitAddUrlHandler(HKEY("upload_file"), upload_file, 0);
+	WebcitAddUrlHandler(HKEY("image"), "", 0, output_image, ANONYMOUS);
+	WebcitAddUrlHandler(HKEY("display_mime_icon"), "", 0, display_mime_icon , ANONYMOUS);
+	WebcitAddUrlHandler(HKEY("download_file"), "", 0, download_file, NEED_URL);
+	WebcitAddUrlHandler(HKEY("delete_file"), "", 0, delete_file, NEED_URL);
+	WebcitAddUrlHandler(HKEY("upload_file"), "", 0, upload_file, 0);
 }
