@@ -1,12 +1,12 @@
 /*
- * $Id: fmt_date.c 7369 2009-04-18 12:27:01Z dothebart $
+ * $Id: fmt_date.c 7716 2009-08-01 10:07:44Z dothebart $
  */
 
 #include "webcit.h"
 #include "webserver.h"
 
 #ifdef HAVE_USELOCALE
-extern locale_t wc_locales[];
+extern locale_t *wc_locales;
 #endif
 
 typedef unsigned char byte;
