@@ -1,5 +1,5 @@
 /*
- * $Id: groupdav_get.c 7077 2009-02-07 11:08:37Z matt $
+ * $Id: groupdav_get.c 7939 2009-10-23 12:51:53Z dothebart $
  *
  * Handles GroupDAV GET requests.
  *
@@ -23,9 +23,11 @@ void groupdav_get_big_ics(void) {
 		hprintf("HTTP/1.1 404 not found\r\n");
 		groupdav_common_headers();
 		hprintf("Content-Type: text/plain\r\n");
-		wprintf("%s\r\n",
+		begin_burst();
+		wc_printf("%s\r\n",
 			&buf[4]
-			);/* TODO: do we need to end-burst here? */
+			);
+		end_burst();
 		return;
 	}
 
@@ -34,7 +36,7 @@ void groupdav_get_big_ics(void) {
 	hprintf("Content-type: text/calendar; charset=UTF-8\r\n");
 	begin_burst();
 	while (serv_getln(buf, sizeof buf), strcmp(buf, "000")) {
-		wprintf("%s\r\n", buf);
+		wc_printf("%s\r\n", buf);
 	}
 	end_burst();
 }
@@ -86,7 +88,9 @@ void extract_preferred(char *name, char *filename, char *partnum, char *disp,
  * /groupdav/room_name/euid	(GroupDAV)
  * /groupdav/room_name		(webcal)
  */
-void groupdav_get(StrBuf *dav_pathname) {
+void groupdav_get(void)
+{
+	wcsession *WCC = WC;
 	StrBuf *dav_roomname;
 	StrBuf *dav_uid;
 	long dav_msgnum = (-1);
@@ -104,33 +108,33 @@ void groupdav_get(StrBuf *dav_pathname) {
 	char date[128];
 	struct epdata epdata;
 
-	if (StrBufNum_tokens(dav_pathname, '/') < 3) {
+	if (StrBufNum_tokens(WCC->Hdr->HR.ReqLine, '/') < 2) {
 		hprintf("HTTP/1.1 404 not found\r\n");
 		groupdav_common_headers();
 		hprintf("Content-Type: text/plain\r\n");
-		wprintf("The object you requested was not found.\r\n");
+		wc_printf("The object you requested was not found.\r\n");
 		end_burst();
 		return;
 	}
 
 	dav_roomname = NewStrBuf();;
 	dav_uid = NewStrBuf();;
-	StrBufExtract_token(dav_roomname, dav_pathname, 2, '/');
-	StrBufExtract_token(dav_uid, dav_pathname, 3, '/');
+	StrBufExtract_token(dav_roomname, WCC->Hdr->HR.ReqLine, 0, '/');
+	StrBufExtract_token(dav_uid, WCC->Hdr->HR.ReqLine, 1, '/');
 	if ((!strcasecmp(ChrPtr(dav_uid), "ics")) || 
 	    (!strcasecmp(ChrPtr(dav_uid), "calendar.ics"))) {
 		FlushStrBuf(dav_uid);
 	}
 
 	/* Go to the correct room. */
-	if (strcasecmp(ChrPtr(WC->wc_roomname), ChrPtr(dav_roomname))) {
+	if (strcasecmp(ChrPtr(WCC->wc_roomname), ChrPtr(dav_roomname))) {
 		gotoroom(dav_roomname);
 	}
-	if (strcasecmp(ChrPtr(WC->wc_roomname), ChrPtr(dav_roomname))) {
+	if (strcasecmp(ChrPtr(WCC->wc_roomname), ChrPtr(dav_roomname))) {
 		hprintf("HTTP/1.1 404 not found\r\n");
 		groupdav_common_headers();
 		hprintf("Content-Type: text/plain\r\n");
-		wprintf("There is no folder called \"%s\" on this server.\r\n",
+		wc_printf("There is no folder called \"%s\" on this server.\r\n",
 			ChrPtr(dav_roomname));
 		end_burst();
 		FreeStrBuf(&dav_roomname);
@@ -154,7 +158,7 @@ void groupdav_get(StrBuf *dav_pathname) {
 		hprintf("HTTP/1.1 404 not found\r\n");
 		groupdav_common_headers();
 		hprintf("Content-Type: text/plain\r\n");
-		wprintf("Object \"%s\" was not found in the \"%s\" folder.\r\n",
+		wc_printf("Object \"%s\" was not found in the \"%s\" folder.\r\n",
 			ChrPtr(dav_uid),
 			ChrPtr(dav_roomname));
 		end_burst();
@@ -226,11 +230,11 @@ void groupdav_get(StrBuf *dav_pathname) {
 	 */
 	if (!strncasecmp(content_type, "multipart/", 10)) {
 
-		if ( (WC->wc_default_view == VIEW_CALENDAR) || (WC->wc_default_view == VIEW_TASKS) ) {
+		if ( (WCC->wc_default_view == VIEW_CALENDAR) || (WCC->wc_default_view == VIEW_TASKS) ) {
 			strcpy(epdata.desired_content_type_1, "text/calendar");
 		}
 
-		else if (WC->wc_default_view == VIEW_ADDRESSBOOK) {
+		else if (WCC->wc_default_view == VIEW_ADDRESSBOOK) {
 			strcpy(epdata.desired_content_type_1, "text/vcard");
 			strcpy(epdata.desired_content_type_2, "text/x-vcard");
 		}
@@ -251,7 +255,7 @@ void groupdav_get(StrBuf *dav_pathname) {
 			ptr = memreadline(ptr, buf, sizeof buf);
 	
 			if (in_body) {
-				wprintf("%s\r\n", buf);
+				wc_printf("%s\r\n", buf);
 			}
 			else if ((buf[0] == 0) && (in_body == 0)) {
 				in_body = 1;
