@@ -1,5 +1,5 @@
 /*
- * $Id: groupdav_main.c 7213 2009-03-12 22:56:24Z dothebart $
+ * $Id: groupdav_main.c 7939 2009-10-23 12:51:53Z dothebart $
  *
  * Entry point for GroupDAV functions
  *
@@ -84,142 +84,89 @@ void euid_unescapize(char *target, const char *source) {
 /*
  * Main entry point for GroupDAV requests
  */
-void groupdav_main(HashList *HTTPHeaders,
-		   StrBuf *DavPathname,
-		   StrBuf *DavMethod,
-		   StrBuf *dav_content_type,
-		   int dav_content_length,
-		   StrBuf *dav_content,
-		   int Offset
-) {
+void groupdav_main(void)
+{
 	wcsession *WCC = WC;
-	void *vLine;
-	char dav_ifmatch[256];
-	int dav_depth;
-	char *ds;
 	int i, len;
 
-	strcpy(dav_ifmatch, "");
-	dav_depth = 0;
+	StrBufUnescape(WCC->Hdr->HR.ReqLine, 0);
 
-	if ((StrLength(WCC->http_host) == 0) &&
-	    GetHash(HTTPHeaders, HKEY("HOST"), &vLine) && 
-	    (vLine != NULL)) {
-		WCC->http_host = (StrBuf*)vLine;
-	}
-	if (GetHash(HTTPHeaders, HKEY("IF-MATCH"), &vLine) && 
-	    (vLine != NULL)) {
-		safestrncpy(dav_ifmatch, ChrPtr((StrBuf*)vLine),
-			    sizeof dav_ifmatch);
-	}
-	if (GetHash(HTTPHeaders, HKEY("DEPTH"), &vLine) && 
-	    (vLine != NULL)) {
-		if (!strcasecmp(ChrPtr((StrBuf*)vLine), "infinity")) {
-			dav_depth = 32767;
-		}
-		else if (strcmp(ChrPtr((StrBuf*)vLine), "0") == 0) {
-			dav_depth = 0;
-		}
-		else if (strcmp(ChrPtr((StrBuf*)vLine), "1") == 0) {
-			dav_depth = 1;
-		}
-	}
-
-	if (!WC->logged_in) {
-		hprintf("HTTP/1.1 401 Unauthorized\r\n");
-		groupdav_common_headers();
-		hprintf("WWW-Authenticate: Basic realm=\"%s\"\r\n",
-			ChrPtr(WCC->serv_info->serv_humannode));
-		hprintf("Content-Length: 0\r\n");
-		end_burst();
-		return;
-	}
-
-	StrBufUnescape(DavPathname, 0);
-
-	/* Remove any stray double-slashes in pathname */
-	while (ds=strstr(ChrPtr(DavPathname), "//"), ds != NULL) {
-		strcpy(ds, ds+1);
-	}
+	StrBufStripSlashes(WCC->Hdr->HR.ReqLine, 0);
 
 	/*
 	 * If there's an If-Match: header, strip out the quotes if present, and
 	 * then if all that's left is an asterisk, make it go away entirely.
 	 */
-	len = strlen(dav_ifmatch);
+	len = StrLength(WCC->Hdr->HR.dav_ifmatch);
 	if (len > 0) {
-		stripltlen(dav_ifmatch, &len);
-		if (dav_ifmatch[0] == '\"') {
-			memmove (dav_ifmatch, &dav_ifmatch[1], len);
+		StrBufTrim(WCC->Hdr->HR.dav_ifmatch);
+		if (ChrPtr(WCC->Hdr->HR.dav_ifmatch)[0] == '\"') {
+			StrBufCutLeft(WCC->Hdr->HR.dav_ifmatch, 1);
 			len --;
 			for (i=0; i<len; ++i) {
-				if (dav_ifmatch[i] == '\"') {
-					dav_ifmatch[i] = 0;
-					len = i - 1;
+				if (ChrPtr(WCC->Hdr->HR.dav_ifmatch)[i] == '\"') {
+					StrBufCutAt(WCC->Hdr->HR.dav_ifmatch, i, NULL);
+					len = StrLength(WCC->Hdr->HR.dav_ifmatch);
 				}
 			}
 		}
-		if (!strcmp(dav_ifmatch, "*")) {
-			strcpy(dav_ifmatch, "");
+		if (!strcmp(ChrPtr(WCC->Hdr->HR.dav_ifmatch), "*")) {
+			FlushStrBuf(WCC->Hdr->HR.dav_ifmatch);
 		}
 	}
 
+	switch (WCC->Hdr->HR.eReqType)
+	{
 	/*
 	 * The OPTIONS method is not required by GroupDAV.  This is an
 	 * experiment to determine what might be involved in supporting
 	 * other variants of DAV in the future.
 	 */
-	if (!strcasecmp(ChrPtr(DavMethod), "OPTIONS")) {
-		groupdav_options(DavPathname);
-		return;
-	}
+	case eOPTIONS:
+		groupdav_options();
+		break;
+
 
 	/*
 	 * The PROPFIND method is basically used to list all objects in a
 	 * room, or to list all relevant rooms on the server.
 	 */
-	if (!strcasecmp(ChrPtr(DavMethod), "PROPFIND")) {
-		groupdav_propfind(DavPathname, dav_depth,
-				  dav_content_type, dav_content, 
-				  Offset);
-		return;
-	}
+	case ePROPFIND:
+		groupdav_propfind();
+		break;
 
 	/*
 	 * The GET method is used for fetching individual items.
 	 */
-	if (!strcasecmp(ChrPtr(DavMethod), "GET")) {
-		groupdav_get(DavPathname);
-		return;
-	}
-
+	case eGET:
+		groupdav_get();
+		break;
+	
 	/*
 	 * The PUT method is used to add or modify items.
 	 */
-	if (!strcasecmp(ChrPtr(DavMethod), "PUT")) {
-		groupdav_put(DavPathname, dav_ifmatch,
-			     ChrPtr(dav_content_type), dav_content, 
-			     Offset);
-		return;
-	}
-
+	case ePUT:
+		groupdav_put();
+		break;
+	
 	/*
 	 * The DELETE method kills, maims, and destroys.
 	 */
-	if (!strcasecmp(ChrPtr(DavMethod), "DELETE")) {
-		groupdav_delete(DavPathname, dav_ifmatch);
-		return;
-	}
+	case eDELETE:
+		groupdav_delete();
+		break;
+	default:
 
 	/*
 	 * Couldn't find what we were looking for.  Die in a car fire.
 	 */
-	hprintf("HTTP/1.1 501 Method not implemented\r\n");
-	groupdav_common_headers();
-	hprintf("Content-Type: text/plain\r\n");
-	wprintf("GroupDAV method \"%s\" is not implemented.\r\n",
-		ChrPtr(DavMethod));
-	end_burst();
+		hprintf("HTTP/1.1 501 Method not implemented\r\n");
+		groupdav_common_headers();
+		hprintf("Content-Type: text/plain\r\n");
+		wc_printf("GroupDAV method \"%s\" is not implemented.\r\n",
+			ReqStrs[WCC->Hdr->HR.eReqType]);
+		end_burst();
+	}
 }
 
 
@@ -229,9 +176,51 @@ void groupdav_main(HashList *HTTPHeaders,
 void groupdav_identify_host(void) {
 	wcsession *WCC = WC;
 
-	if (StrLength(WCC->http_host)!=0) {
-		wprintf("%s://%s",
+	if (StrLength(WCC->Hdr->HR.http_host)!=0) {
+		wc_printf("%s://%s",
 			(is_https ? "https" : "http"),
-			ChrPtr(WCC->http_host));
+			ChrPtr(WCC->Hdr->HR.http_host));
 	}
 }
+
+/*
+ * Output our host prefix for globally absolute URL's.
+ */  
+void groupdav_identify_hosthdr(void) {
+	wcsession *WCC = WC;
+
+	if (StrLength(WCC->Hdr->HR.http_host)!=0) {
+		hprintf("%s://%s",
+			(is_https ? "https" : "http"),
+			ChrPtr(WCC->Hdr->HR.http_host));
+	}
+}
+
+
+void Header_HandleIfMatch(StrBuf *Line, ParsedHttpHdrs *hdr)
+{
+	hdr->HR.dav_ifmatch = Line;
+}
+	
+void Header_HandleDepth(StrBuf *Line, ParsedHttpHdrs *hdr)
+{
+	if (!strcasecmp(ChrPtr(Line), "infinity")) {
+		hdr->HR.dav_depth = 32767;
+	}
+	else if (strcmp(ChrPtr(Line), "0") == 0) {
+		hdr->HR.dav_depth = 0;
+	}
+	else if (strcmp(ChrPtr(Line), "1") == 0) {
+		hdr->HR.dav_depth = 1;
+	}
+}
+
+void 
+InitModule_GROUPDAV
+(void)
+{
+	WebcitAddUrlHandler(HKEY("groupdav"), "", 0, groupdav_main, XHTTP_COMMANDS|COOKIEUNNEEDED|FORCE_SESSIONCLOSE);
+	RegisterHeaderHandler(HKEY("IF-MATCH"), Header_HandleIfMatch);
+	RegisterHeaderHandler(HKEY("DEPTH"), Header_HandleDepth);
+
+}
