@@ -1,13 +1,12 @@
 /*
- * $Id: groupdav_propfind.c 7086 2009-02-10 03:59:41Z ajc $
+ * $Id: groupdav_propfind.c 7939 2009-10-23 12:51:53Z dothebart $
  *
  * Handles GroupDAV PROPFIND requests.
  *
  * A few notes about our XML output:
  *
  * --> Yes, we are spewing tags directly instead of using an XML library.
- *     If you would like to rewrite this using libxml2, code it up and submit
- *     a patch.  Whining will be summarily ignored.
+ *     Whining about it will be summarily ignored.
  *
  * --> XML is deliberately output with no whitespace/newlines between tags.
  *     This makes it difficult to read, but we have discovered clients which
@@ -49,8 +48,9 @@ long locate_message_by_uid(const char *uid) {
  * List rooms (or "collections" in DAV terminology) which contain
  * interesting groupware objects.
  */
-void groupdav_collection_list(const char *dav_pathname, int dav_depth)
+void groupdav_collection_list(void)
 {
+	wcsession *WCC = WC;
 	char buf[256];
 	char roomname[256];
 	int view;
@@ -60,23 +60,20 @@ void groupdav_collection_list(const char *dav_pathname, int dav_depth)
 	int is_groupware_collection = 0;
 	int starting_point = 1;		/**< 0 for /, 1 for /groupdav/ */
 
-	if (!strcmp(dav_pathname, "/")) {
+	if (WCC->Hdr->HR.Handler == NULL) {
 		starting_point = 0;
 	}
-	else if (!strcasecmp(dav_pathname, "/groupdav")) {
+	else if (StrLength(WCC->Hdr->HR.ReqLine) == 0) {
 		starting_point = 1;
 	}
-	else if (!strcasecmp(dav_pathname, "/groupdav/")) {
-		starting_point = 1;
-	}
-	else if ( (!strncasecmp(dav_pathname, "/groupdav/", 10)) && (strlen(dav_pathname) > 10) ) {
+	else {
 		starting_point = 2;
 	}
 
 	now = time(NULL);
 	http_datestring(datestring, sizeof datestring, now);
 
-	/**
+	/*
 	 * Be rude.  Completely ignore the XML request and simply send them
 	 * everything we know about.  Let the client sort it out.
 	 */
@@ -88,56 +85,56 @@ void groupdav_collection_list(const char *dav_pathname, int dav_depth)
 
 	begin_burst();
 
-	wprintf("<?xml version=\"1.0\" encoding=\"utf-8\"?>"
+	wc_printf("<?xml version=\"1.0\" encoding=\"utf-8\"?>"
      		"<multistatus xmlns=\"DAV:\" xmlns:G=\"http://groupdav.org/\">"
 	);
 
-	/**
-	 *	If the client is requesting the root, show a root node.
+	/*
+	 * If the client is requesting the root, show a root node.
 	 */
 	if (starting_point == 0) {
-		wprintf("<response>");
-			wprintf("<href>");
+		wc_printf("<response>");
+			wc_printf("<href>");
 				groupdav_identify_host();
-				wprintf("/");
-			wprintf("</href>");
-			wprintf("<propstat>");
-				wprintf("<status>HTTP/1.1 200 OK</status>");
-				wprintf("<prop>");
-					wprintf("<displayname>/</displayname>");
-					wprintf("<resourcetype><collection/></resourcetype>");
-					wprintf("<getlastmodified>");
+				wc_printf("/");
+			wc_printf("</href>");
+			wc_printf("<propstat>");
+				wc_printf("<status>HTTP/1.1 200 OK</status>");
+				wc_printf("<prop>");
+					wc_printf("<displayname>/</displayname>");
+					wc_printf("<resourcetype><collection/></resourcetype>");
+					wc_printf("<getlastmodified>");
 						escputs(datestring);
-					wprintf("</getlastmodified>");
-				wprintf("</prop>");
-			wprintf("</propstat>");
-		wprintf("</response>");
+					wc_printf("</getlastmodified>");
+				wc_printf("</prop>");
+			wc_printf("</propstat>");
+		wc_printf("</response>");
 	}
 
-	/**
-	 *	If the client is requesting "/groupdav", show a /groupdav subdirectory.
+	/*
+	 * If the client is requesting "/groupdav", show a /groupdav subdirectory.
 	 */
-	if ((starting_point + dav_depth) >= 1) {
-		wprintf("<response>");
-			wprintf("<href>");
+	if ((starting_point + WCC->Hdr->HR.dav_depth) >= 1) {
+		wc_printf("<response>");
+			wc_printf("<href>");
 				groupdav_identify_host();
-				wprintf("/groupdav");
-			wprintf("</href>");
-			wprintf("<propstat>");
-				wprintf("<status>HTTP/1.1 200 OK</status>");
-				wprintf("<prop>");
-					wprintf("<displayname>GroupDAV</displayname>");
-					wprintf("<resourcetype><collection/></resourcetype>");
-					wprintf("<getlastmodified>");
+				wc_printf("/groupdav");
+			wc_printf("</href>");
+			wc_printf("<propstat>");
+				wc_printf("<status>HTTP/1.1 200 OK</status>");
+				wc_printf("<prop>");
+					wc_printf("<displayname>GroupDAV</displayname>");
+					wc_printf("<resourcetype><collection/></resourcetype>");
+					wc_printf("<getlastmodified>");
 						escputs(datestring);
-					wprintf("</getlastmodified>");
-				wprintf("</prop>");
-			wprintf("</propstat>");
-		wprintf("</response>");
+					wc_printf("</getlastmodified>");
+				wc_printf("</prop>");
+			wc_printf("</propstat>");
+		wc_printf("</response>");
 	}
 
-	/**
-	 *	Now go through the list and make it look like a DAV collection
+	/*
+	 * Now go through the list and make it look like a DAV collection
 	 */
 	serv_puts("LKRA");
 	serv_getln(buf, sizeof buf);
@@ -159,52 +156,67 @@ void groupdav_collection_list(const char *dav_pathname, int dav_depth)
 		 * GroupDAV calendar even if the user has switched it to a
 		 * Calendar List view.
 		 */
-		if ((view == VIEW_CALENDAR) || (view == VIEW_TASKS) || (view == VIEW_ADDRESSBOOK) ) {
+		if (	(view == VIEW_CALENDAR) || 
+			(view == VIEW_TASKS) || 
+			(view == VIEW_ADDRESSBOOK) ||
+			(view == VIEW_NOTES) ||
+			(view == VIEW_JOURNAL) ||
+			(view == VIEW_WIKI)
+		) {
 			is_groupware_collection = 1;
 		}
 		else {
 			is_groupware_collection = 0;
 		}
 
-		if ( (is_groupware_collection) && ((starting_point + dav_depth) >= 2) ) {
-			wprintf("<response>");
+		if ( (is_groupware_collection) && ((starting_point + WCC->Hdr->HR.dav_depth) >= 2) ) {
+			wc_printf("<response>");
 
-			wprintf("<href>");
+			wc_printf("<href>");
 			groupdav_identify_host();
-			wprintf("/groupdav/");
+			wc_printf("/groupdav/");
 			urlescputs(roomname);
-			wprintf("/</href>");
+			wc_printf("/</href>");
 
-			wprintf("<propstat>");
-			wprintf("<status>HTTP/1.1 200 OK</status>");
-			wprintf("<prop>");
-			wprintf("<displayname>");
+			wc_printf("<propstat>");
+			wc_printf("<status>HTTP/1.1 200 OK</status>");
+			wc_printf("<prop>");
+			wc_printf("<displayname>");
 			escputs(roomname);
-			wprintf("</displayname>");
-			wprintf("<resourcetype><collection/>");
+			wc_printf("</displayname>");
+			wc_printf("<resourcetype><collection/>");
 
 			switch(view) {
-				case VIEW_CALENDAR:
-					wprintf("<G:vevent-collection />");
-					break;
-				case VIEW_TASKS:
-					wprintf("<G:vtodo-collection />");
-					break;
-				case VIEW_ADDRESSBOOK:
-					wprintf("<G:vcard-collection />");
-					break;
+			case VIEW_CALENDAR:
+				wc_printf("<G:vevent-collection />");
+				break;
+			case VIEW_TASKS:
+				wc_printf("<G:vtodo-collection />");
+				break;
+			case VIEW_ADDRESSBOOK:
+				wc_printf("<G:vcard-collection />");
+				break;
+			case VIEW_NOTES:
+				wc_printf("<G:vnotes-collection />");
+				break;
+			case VIEW_JOURNAL:
+				wc_printf("<G:vjournal-collection />");
+				break;
+			case VIEW_WIKI:
+				wc_printf("<G:wiki-collection />");
+				break;
 			}
 
-			wprintf("</resourcetype>");
-			wprintf("<getlastmodified>");
+			wc_printf("</resourcetype>");
+			wc_printf("<getlastmodified>");
 				escputs(datestring);
-			wprintf("</getlastmodified>");
-			wprintf("</prop>");
-			wprintf("</propstat>");
-			wprintf("</response>");
+			wc_printf("</getlastmodified>");
+			wc_printf("</prop>");
+			wc_printf("</propstat>");
+			wc_printf("</response>");
 		}
 	}
-	wprintf("</multistatus>\n");
+	wc_printf("</multistatus>\n");
 
 	end_burst();
 }
@@ -214,12 +226,14 @@ void groupdav_collection_list(const char *dav_pathname, int dav_depth)
 /*
  * The pathname is always going to be /groupdav/room_name/msg_num
  */
-void groupdav_propfind(StrBuf *dav_pathname, int dav_depth, StrBuf *dav_content_type, StrBuf *dav_content, int offset) {
+void groupdav_propfind(void) 
+{
+	wcsession *WCC = WC;
 	StrBuf *dav_roomname;
 	StrBuf *dav_uid;
-	char msgnum[256];
+	StrBuf *MsgNum;
+	long BufLen;
 	long dav_msgnum = (-1);
-	char buf[256];
 	char uid[256];
 	char encoded_uid[256];
 	long *msgs = NULL;
@@ -233,30 +247,30 @@ void groupdav_propfind(StrBuf *dav_pathname, int dav_depth, StrBuf *dav_content_
 
 	dav_roomname = NewStrBuf();
 	dav_uid = NewStrBuf();
-	StrBufExtract_token(dav_roomname, dav_pathname, 2, '/');
-	StrBufExtract_token(dav_uid, dav_pathname, 3, '/');
+	StrBufExtract_token(dav_roomname, WCC->Hdr->HR.ReqLine, 0, '/');
+	StrBufExtract_token(dav_uid, WCC->Hdr->HR.ReqLine, 1, '/');
 
 	/*
 	 * If the room name is blank, the client is requesting a
 	 * folder list.
 	 */
 	if (StrLength(dav_roomname) == 0) {
-		groupdav_collection_list(ChrPtr(dav_pathname), dav_depth);
+		groupdav_collection_list();
 		FreeStrBuf(&dav_roomname);
 		FreeStrBuf(&dav_uid);
 		return;
 	}
 
 	/* Go to the correct room. */
-	if (strcasecmp(ChrPtr(WC->wc_roomname), ChrPtr(dav_roomname))) {
+	if (strcasecmp(ChrPtr(WCC->wc_roomname), ChrPtr(dav_roomname))) {
 		gotoroom(dav_roomname);
 	}
-	if (strcasecmp(ChrPtr(WC->wc_roomname), ChrPtr(dav_roomname))) {
+	if (strcasecmp(ChrPtr(WCC->wc_roomname), ChrPtr(dav_roomname))) {
 		hprintf("HTTP/1.1 404 not found\r\n");
 		groupdav_common_headers();
 		hprintf("Date: %s\r\n", datestring);
 		hprintf("Content-Type: text/plain\r\n");
-		wprintf("There is no folder called \"%s\" on this server.\r\n",
+		wc_printf("There is no folder called \"%s\" on this server.\r\n",
 			ChrPtr(dav_roomname)
 		);
 		end_burst();
@@ -276,7 +290,7 @@ void groupdav_propfind(StrBuf *dav_pathname, int dav_depth, StrBuf *dav_content_
 			hprintf("HTTP/1.1 404 not found\r\n");
 			groupdav_common_headers();
 			hprintf("Content-Type: text/plain\r\n");
-			wprintf("Object \"%s\" was not found in the \"%s\" folder.\r\n",
+			wc_printf("Object \"%s\" was not found in the \"%s\" folder.\r\n",
 				ChrPtr(dav_uid),
 				ChrPtr(dav_roomname)
 			);
@@ -298,31 +312,31 @@ void groupdav_propfind(StrBuf *dav_pathname, int dav_depth, StrBuf *dav_content_
 	
 		begin_burst();
 	
-		wprintf("<?xml version=\"1.0\" encoding=\"utf-8\"?>"
+		wc_printf("<?xml version=\"1.0\" encoding=\"utf-8\"?>"
      			"<multistatus xmlns=\"DAV:\">"
 		);
 
-		wprintf("<response>");
+		wc_printf("<response>");
 		
-		wprintf("<href>");
+		wc_printf("<href>");
 		groupdav_identify_host();
-		wprintf("/groupdav/");
-		urlescputs(ChrPtr(WC->wc_roomname));
+		wc_printf("/groupdav/");
+		urlescputs(ChrPtr(WCC->wc_roomname));
 		euid_escapize(encoded_uid, ChrPtr(dav_uid));
-		wprintf("/%s", encoded_uid);
-		wprintf("</href>");
-		wprintf("<propstat>");
-		wprintf("<status>HTTP/1.1 200 OK</status>");
-		wprintf("<prop>");
-		wprintf("<getetag>\"%ld\"</getetag>", dav_msgnum);
-		wprintf("<getlastmodified>");
+		wc_printf("/%s", encoded_uid);
+		wc_printf("</href>");
+		wc_printf("<propstat>");
+		wc_printf("<status>HTTP/1.1 200 OK</status>");
+		wc_printf("<prop>");
+		wc_printf("<getetag>\"%ld\"</getetag>", dav_msgnum);
+		wc_printf("<getlastmodified>");
 		escputs(datestring);
-		wprintf("</getlastmodified>");
-		wprintf("</prop>");
-		wprintf("</propstat>");
+		wc_printf("</getlastmodified>");
+		wc_printf("</prop>");
+		wc_printf("</propstat>");
 
-		wprintf("</response>\n");
-		wprintf("</multistatus>\n");
+		wc_printf("</response>\n");
+		wc_printf("</multistatus>\n");
 		end_burst();
 		FreeStrBuf(&dav_roomname);
 		FreeStrBuf(&dav_uid);
@@ -348,111 +362,117 @@ void groupdav_propfind(StrBuf *dav_pathname, int dav_depth, StrBuf *dav_content_
 
 	begin_burst();
 
-	wprintf("<?xml version=\"1.0\" encoding=\"utf-8\"?>"
+	wc_printf("<?xml version=\"1.0\" encoding=\"utf-8\"?>"
      		"<multistatus xmlns=\"DAV:\" xmlns:G=\"http://groupdav.org/\">"
 	);
 
 
-	/** Transmit the collection resource (FIXME check depth and starting point) */
-	wprintf("<response>");
+	/* Transmit the collection resource (FIXME check depth and starting point) */
+	wc_printf("<response>");
 
-	wprintf("<href>");
-		groupdav_identify_host();
-		wprintf("/groupdav/");
-		urlescputs(ChrPtr(WC->wc_roomname));
-	wprintf("</href>");
+	wc_printf("<href>");
+	groupdav_identify_host();
+	wc_printf("/groupdav/");
+	urlescputs(ChrPtr(WCC->wc_roomname));
+	wc_printf("</href>");
 
-	wprintf("<propstat>");
-	wprintf("<status>HTTP/1.1 200 OK</status>");
-	wprintf("<prop>");
-	wprintf("<displayname>");
-	escputs(ChrPtr(WC->wc_roomname));
-	wprintf("</displayname>");
-	wprintf("<resourcetype><collection/>");
+	wc_printf("<propstat>");
+	wc_printf("<status>HTTP/1.1 200 OK</status>");
+	wc_printf("<prop>");
+	wc_printf("<displayname>");
+	escputs(ChrPtr(WCC->wc_roomname));
+	wc_printf("</displayname>");
+	wc_printf("<resourcetype><collection/>");
 
-	switch(WC->wc_default_view) {
+	switch(WCC->wc_default_view) {
 		case VIEW_CALENDAR:
-			wprintf("<G:vevent-collection />");
+			wc_printf("<G:vevent-collection />");
 			break;
 		case VIEW_TASKS:
-			wprintf("<G:vtodo-collection />");
+			wc_printf("<G:vtodo-collection />");
 			break;
 		case VIEW_ADDRESSBOOK:
-			wprintf("<G:vcard-collection />");
+			wc_printf("<G:vcard-collection />");
 			break;
 	}
 
-	wprintf("</resourcetype>");
+	wc_printf("</resourcetype>");
 	/* FIXME get the mtime
-	wprintf("<getlastmodified>");
+	wc_printf("<getlastmodified>");
 		escputs(datestring);
-	wprintf("</getlastmodified>");
+	wc_printf("</getlastmodified>");
 	*/
-	wprintf("</prop>");
-	wprintf("</propstat>");
-	wprintf("</response>");
+	wc_printf("</prop>");
+	wc_printf("</propstat>");
+	wc_printf("</response>");
 
-	/** Transmit the collection listing (FIXME check depth and starting point) */
+	/* Transmit the collection listing (FIXME check depth and starting point) */
 
+	MsgNum = NewStrBuf();
 	serv_puts("MSGS ALL");
-	serv_getln(buf, sizeof buf);
-	if (buf[0] == '1') while (serv_getln(msgnum, sizeof msgnum), strcmp(msgnum, "000")) {
-		msgs = realloc(msgs, ++num_msgs * sizeof(long));
-		msgs[num_msgs-1] = atol(msgnum);
-	}
+
+	StrBuf_ServGetln(MsgNum);
+	if (GetServerStatus(MsgNum, NULL) == 1)
+		while (BufLen = StrBuf_ServGetln(MsgNum), strcmp(ChrPtr(MsgNum), "000"))  {
+			msgs = realloc(msgs, ++num_msgs * sizeof(long));
+			msgs[num_msgs-1] = StrTol(MsgNum);
+		}
 
 	if (num_msgs > 0) for (i=0; i<num_msgs; ++i) {
 
 		strcpy(uid, "");
 		now = (-1);
 		serv_printf("MSG0 %ld|3", msgs[i]);
-		serv_getln(buf, sizeof buf);
-		if (buf[0] == '1') while (serv_getln(buf, sizeof buf), strcmp(buf, "000")) {
-			if (!strncasecmp(buf, "exti=", 5)) {
-				strcpy(uid, &buf[5]);
-			}
-			else if (!strncasecmp(buf, "time=", 5)) {
-				now = atol(&buf[5]);
+		StrBuf_ServGetln(MsgNum);
+		if (GetServerStatus(MsgNum, NULL) == 1)
+			while (BufLen = StrBuf_ServGetln(MsgNum), strcmp(ChrPtr(MsgNum), "000")) 
+			{
+				if (!strncasecmp(ChrPtr(MsgNum), "exti=", 5)) {
+					strcpy(uid, &ChrPtr(MsgNum)[5]);
+				}
+				else if (!strncasecmp(ChrPtr(MsgNum), "time=", 5)) {
+					now = atol(&ChrPtr(MsgNum)[5]);
 			}
 		}
 
 		if (!IsEmptyStr(uid)) {
-			wprintf("<response>");
-				wprintf("<href>");
+			wc_printf("<response>");
+				wc_printf("<href>");
 					groupdav_identify_host();
-					wprintf("/groupdav/");
-					urlescputs(ChrPtr(WC->wc_roomname));
+					wc_printf("/groupdav/");
+					urlescputs(ChrPtr(WCC->wc_roomname));
 					euid_escapize(encoded_uid, uid);
-					wprintf("/%s", encoded_uid);
-				wprintf("</href>");
-				switch(WC->wc_default_view) {
+					wc_printf("/%s", encoded_uid);
+				wc_printf("</href>");
+				switch(WCC->wc_default_view) {
 				case VIEW_CALENDAR:
-					wprintf("<getcontenttype>text/x-ical</getcontenttype>");
+					wc_printf("<getcontenttype>text/x-ical</getcontenttype>");
 					break;
 				case VIEW_TASKS:
-					wprintf("<getcontenttype>text/x-ical</getcontenttype>");
+					wc_printf("<getcontenttype>text/x-ical</getcontenttype>");
 					break;
 				case VIEW_ADDRESSBOOK:
-					wprintf("<getcontenttype>text/x-vcard</getcontenttype>");
+					wc_printf("<getcontenttype>text/x-vcard</getcontenttype>");
 					break;
 				}
-				wprintf("<propstat>");
-					wprintf("<status>HTTP/1.1 200 OK</status>");
-					wprintf("<prop>");
-						wprintf("<getetag>\"%ld\"</getetag>", msgs[i]);
+				wc_printf("<propstat>");
+					wc_printf("<status>HTTP/1.1 200 OK</status>");
+					wc_printf("<prop>");
+						wc_printf("<getetag>\"%ld\"</getetag>", msgs[i]);
 					if (now > 0L) {
 						http_datestring(datestring, sizeof datestring, now);
-						wprintf("<getlastmodified>");
+						wc_printf("<getlastmodified>");
 						escputs(datestring);
-						wprintf("</getlastmodified>");
+						wc_printf("</getlastmodified>");
 					}
-					wprintf("</prop>");
-				wprintf("</propstat>");
-			wprintf("</response>");
+					wc_printf("</prop>");
+				wc_printf("</propstat>");
+			wc_printf("</response>");
 		}
 	}
+	FreeStrBuf(&MsgNum);
 
-	wprintf("</multistatus>\n");
+	wc_printf("</multistatus>\n");
 	end_burst();
 
 	if (msgs != NULL) {
