@@ -1,5 +1,5 @@
 /*
- * $Id: html2html.c 7388 2009-04-22 02:25:02Z ajc $
+ * $Id: html2html.c 8086 2009-11-25 21:45:26Z ajc $
  */
 /**
  * \defgroup HTML2HTML Output an HTML message, modifying it slightly to make sure it plays nice
@@ -369,7 +369,7 @@ void output_html(const char *supplied_charset, int treat_as_wiki, int msgnum, St
 		if (!strncasecmp(ptr, "<a href=\"mailto:", 16)) {
 			content_length += 64;
 			StrBufAppendPrintf(converted_msg,
-					"<a href=\"display_enter?force_room=_MAIL_&recp=");
+					"<a href=\"display_enter?force_room=_MAIL_?recp=");
 			ptr = &ptr[16];
 			++alevel;
 			++brak;
@@ -397,9 +397,19 @@ void output_html(const char *supplied_charset, int treat_as_wiki, int msgnum, St
 		}
 		/** Fixup <img src="cid:... ...> to fetch the mime part */
 		else if (!strncasecmp(ptr, "<img ", 5)) {
-			char* tag_end=strchr(ptr,'>');
-			char* src=strstr(ptr, " src=\"cid:");
 			char *cid_start, *cid_end;
+			char* tag_end=strchr(ptr,'>');
+			char* src;
+			/* FIXME - handle this situation (maybe someone opened an <img cid... 
+			 * and then ended the message)
+			 */
+			if (!tag_end) {
+				lprintf(9, "tag_end is null and ptr is:\n");
+				lprintf(9, "%s\n", ptr);
+				lprintf(9, "Theoretical bytes remaining: %d\n", msgend - ptr);
+			}
+
+			src=strstr(ptr, " src=\"cid:");
 			++brak;
 
 			if (src
