@@ -1,5 +1,5 @@
 /*
- * $Id: paging.c 7231 2009-03-19 04:32:15Z ajc $
+ * $Id: paging.c 7939 2009-10-23 12:51:53Z dothebart $
  */
 
 #include "webcit.h"
@@ -14,44 +14,44 @@ void display_page(void)
 	strcpy(recp, bstr("recp"));
 
         output_headers(1, 1, 2, 0, 0, 0);
-        wprintf("<div id=\"banner\">\n");
-        wprintf("<h1>");
-	wprintf(_("Send instant message"));
-	wprintf("</h1>");
-        wprintf("</div>\n");
+        wc_printf("<div id=\"banner\">\n");
+        wc_printf("<h1>");
+	wc_printf(_("Send instant message"));
+	wc_printf("</h1>");
+        wc_printf("</div>\n");
 
-	wprintf("<div id=\"content\" class=\"service\">\n");
+	wc_printf("<div id=\"content\" class=\"service\">\n");
 
-        wprintf("<div class=\"fix_scrollbar_bug\">"
+        wc_printf("<div class=\"fix_scrollbar_bug\">"
 		"<table class=\"paging_background\"><tr><td>\n");
 
-	wprintf(_("Send an instant message to: "));
+	wc_printf(_("Send an instant message to: "));
 	escputs(recp);
-	wprintf("<br>\n");
+	wc_printf("<br>\n");
 
-	wprintf("<FORM METHOD=\"POST\" action=\"page_user\">\n");
-	wprintf("<input type=\"hidden\" name=\"nonce\" value=\"%d\">\n", WC->nonce);
-	wprintf("<input type=\"hidden\" name=\"template\" value=\"who\">\n");
+	wc_printf("<FORM METHOD=\"POST\" action=\"page_user\">\n");
+	wc_printf("<input type=\"hidden\" name=\"nonce\" value=\"%d\">\n", WC->nonce);
+	wc_printf("<input type=\"hidden\" name=\"template\" value=\"who\">\n");
 
-	wprintf("<TABLE border=0 width=100%%><TR><TD>\n");
+	wc_printf("<TABLE border=0 width=100%%><TR><TD>\n");
 
-	wprintf("<INPUT TYPE=\"hidden\" NAME=\"recp\" VALUE=\"");
+	wc_printf("<INPUT TYPE=\"hidden\" NAME=\"recp\" VALUE=\"");
 	escputs(recp);
-	wprintf("\">\n");
+	wc_printf("\">\n");
 
-	wprintf(_("Enter message text:"));
-	wprintf("<br />");
+	wc_printf(_("Enter message text:"));
+	wc_printf("<br />");
 
-	wprintf("<TEXTAREA NAME=\"msgtext\" wrap=soft ROWS=5 COLS=40 "
+	wc_printf("<TEXTAREA NAME=\"msgtext\" wrap=soft ROWS=5 COLS=40 "
 		"WIDTH=40></TEXTAREA>\n");
 
-	wprintf("</TD></TR></TABLE><br />\n");
+	wc_printf("</TD></TR></TABLE><br />\n");
 
-	wprintf("<INPUT TYPE=\"submit\" NAME=\"send_button\" VALUE=\"%s\">", _("Send message"));
-	wprintf("<br /><a href=\"javascript:window.close();\"%s</A>\n", _("Cancel"));
+	wc_printf("<INPUT TYPE=\"submit\" NAME=\"send_button\" VALUE=\"%s\">", _("Send message"));
+	wc_printf("<br /><a href=\"javascript:window.close();\"%s</A>\n", _("Cancel"));
 
-	wprintf("</FORM></CENTER>\n");
-	wprintf("</td></tr></table></div>\n");
+	wc_printf("</FORM></CENTER>\n");
+	wc_printf("</td></tr></table></div>\n");
 	wDumpContent(1);
 }
 
@@ -139,58 +139,22 @@ void do_chat(void)
  * If there are instant messages waiting, and we notice that we haven't checked them in
  * a while, it probably means that we need to open the instant messenger window.
  */
-void page_popup(void)
+int Conditional_PAGE_WAITING(StrBuf *Target, WCTemplputParams *TP)
 {
 	int len;
 	char buf[SIZ];
 
 	/** JavaScript function to alert the user that popups are probably blocked */
-	wprintf("<script type=\"text/javascript\">	"
-		"function PopUpFailed() {	"
-		" alert(\"%s\");	"
-		"}	"
-		"</script>\n",
-		_("You have one or more instant messages waiting, but the Citadel Instant Messenger "
-		  "window failed to open.  This is probably because you have a popup blocker "
-		  "installed.  Please configure your popup blocker to allow popups from this site "
-		  "if you wish to receive instant messages.")
-	);
-
 	/** First, do the check as part of our page load. */
 	serv_puts("NOOP");
 	len = serv_getln(buf, sizeof buf);
 	if ((len >= 3) && (buf[3] == '*')) {
 		if ((time(NULL) - WC->last_pager_check) > 60) {
-			wprintf("<script type=\"text/javascript\">"
-				" var oWin = window.open('static/instant_messenger.html', "
-				" 'CTDL_MESSENGER', 'width=700,height=400');	"
-				" if (oWin==null || typeof(oWin)==\"undefined\") {	"
-				"  PopUpFailed();	"
-				" }	"
-				"</script>"
-			);	
+			return 1;
 		}
 	}
-
+	return 0;
 	/** Then schedule it to happen again a minute from now if the user is idle. */
-	wprintf("<script type=\"text/javascript\">	"
-		" function HandleSslp(sslg_xmlresponse) {	"
-		"  sslg_response = sslg_xmlresponse.responseText.substr(0, 1);	"
-		"  if (sslg_response == 'Y') {	"
-		"   var oWin = window.open('static/instant_messenger.html', 'CTDL_MESSENGER',	"
-		"    'width=700,height=400');	"
-		"   if (oWin==null || typeof(oWin)==\"undefined\") {	"
-		"    PopUpFailed();	"
-		"   }	"
-		"  }	"
-		" }	"
-		" function CheckPager() {	"
-		"  new Ajax.Request('sslg', { method: 'get', parameters: CtdlRandomString(),	"
-		"   onSuccess: HandleSslp } );	"
-		" }	"
-		" new PeriodicalExecuter(CheckPager, 30);	"
-		"</script>	"
-	);
 }
 
 
@@ -279,7 +243,8 @@ void chat_recv(void) {
 	output_headers(0, 0, 0, 0, 0, 0);
 
 	hprintf("Content-type: text/html; charset=utf-8\r\n");
-	wprintf("<html>\n"
+	begin_burst();
+	wc_printf("<html>\n"
 		"<head>\n"
 		"<meta http-equiv=\"refresh\" content=\"3\" />\n"
 		"</head>\n"
@@ -288,8 +253,8 @@ void chat_recv(void) {
 	);
 
 	if (setup_chat_socket() != 0) {
-		wprintf(_("An error occurred while setting up the chat socket."));
-		wprintf("</BODY></HTML>\n");
+		wc_printf(_("An error occurred while setting up the chat socket."));
+		wc_printf("</BODY></HTML>\n");
 		wDumpContent(0);
 		return;
 	}
@@ -335,7 +300,7 @@ void chat_recv(void) {
 	if (end_chat_now) {
 		close(WC->chat_sock);
 		WC->chat_sock = (-1);
-		wprintf("<img src=\"static/blank.gif\" onLoad=\"parent.window.close();\">\n");
+		wc_printf("<img src=\"static/blank.gif\" onLoad=\"parent.window.close();\">\n");
 	}
 
 	if (!IsEmptyStr(output_data)) {
@@ -346,7 +311,7 @@ void chat_recv(void) {
 		}
 
 		/** Output our fun to the other frame. */
-		wprintf("<img src=\"static/blank.gif\" WIDTH=1 HEIGHT=1\n"
+		wc_printf("<img src=\"static/blank.gif\" WIDTH=1 HEIGHT=1\n"
 			"onLoad=\" \n"
 		);
 
@@ -357,10 +322,10 @@ void chat_recv(void) {
 
 			if (strcasecmp(cl_text, "NOOP")) {
 
-				wprintf("parent.chat_transcript.document.write('");
+				wc_printf("parent.chat_transcript.document.write('");
 	
 				if (strcasecmp(cl_user, WC->last_chat_user)) {
-					wprintf("<TABLE border=0 WIDTH=100%% "
+					wc_printf("<TABLE border=0 WIDTH=100%% "
 						"CELLSPACING=1 CELLPADDING=0 "
 						"BGCOLOR=&quot;#FFFFFF&quot;>"
 						"<TR><TD></TR></TD></TABLE>"
@@ -368,50 +333,50 @@ void chat_recv(void) {
 	
 				}
 
-				wprintf("<TABLE border=0 WIDTH=100%% "
+				wc_printf("<TABLE border=0 WIDTH=100%% "
 					"CELLSPACING=0 CELLPADDING=0 "
 					"BGCOLOR=&quot;#EEEEEE&quot;>");
 	
-				wprintf("<TR><TD>");
+				wc_printf("<TR><TD>");
 	
 				if (!strcasecmp(cl_user, ":")) {
-					wprintf("<I>");
+					wc_printf("<I>");
 				}
 
 				if (strcasecmp(cl_user, WC->last_chat_user)) {
-					wprintf("<B>");
+					wc_printf("<B>");
 	
 					if (!strcasecmp(cl_user, ChrPtr(WC->wc_fullname))) {
-						wprintf("<FONT COLOR=&quot;#FF0000&quot;>");
+						wc_printf("<FONT COLOR=&quot;#FF0000&quot;>");
 					}
 					else {
-						wprintf("<FONT COLOR=&quot;#0000FF&quot;>");
+						wc_printf("<FONT COLOR=&quot;#0000FF&quot;>");
 					}
 					jsescputs(cl_user);
 	
-					wprintf("</FONT>: </B>");
+					wc_printf("</FONT>: </B>");
 				}
 				else {
-					wprintf("&nbsp;&nbsp;&nbsp;");
+					wc_printf("&nbsp;&nbsp;&nbsp;");
 				}
 				jsescputs(cl_text);
 				if (!strcasecmp(cl_user, ":")) {
-					wprintf("</I>");
+					wc_printf("</I>");
 				}
 
-				wprintf("</TD></TR></TABLE>");
-				wprintf("'); \n");
+				wc_printf("</TD></TR></TABLE>");
+				wc_printf("'); \n");
 
 				strcpy(WC->last_chat_user, cl_user);
 			}
 		}
 
-		wprintf("parent.chat_transcript.scrollTo(999999,999999);\">\n");
+		wc_printf("parent.chat_transcript.scrollTo(999999,999999);\">\n");
 	}
 
 	free(output_data);
 
-	wprintf("</BODY></HTML>\n");
+	wc_printf("</BODY></HTML>\n");
 	wDumpContent(0);
 }
 
@@ -426,7 +391,8 @@ void chat_send(void) {
 
 	output_headers(0, 0, 0, 0, 0, 0);
 	hprintf("Content-type: text/html; charset=utf-8\r\n");
-	wprintf("<HTML>"
+	begin_burst();
+	wc_printf("<HTML>"
 		"<BODY onLoad=\"document.chatsendform.send_this.focus();\" >"
 	);
 
@@ -450,8 +416,8 @@ void chat_send(void) {
 	}
 
 	if (setup_chat_socket() != 0) {
-		wprintf(_("An error occurred while setting up the chat socket."));
-		wprintf("</BODY></HTML>\n");
+		wc_printf(_("An error occurred while setting up the chat socket."));
+		wc_printf("</BODY></HTML>\n");
 		wDumpContent(0);
 		return;
 	}
@@ -482,18 +448,18 @@ void chat_send(void) {
 	WC->serv_sock = WC->chat_sock;
 	WC->chat_sock = i;
 
-	wprintf("<FORM METHOD=\"POST\" action=\"chat_send\" NAME=\"chatsendform\">\n");
-	wprintf("<input type=\"hidden\" name=\"nonce\" value=\"%d\">\n", WC->nonce);
-	wprintf("<INPUT TYPE=\"text\" SIZE=\"80\" MAXLENGTH=\"%d\" "
+	wc_printf("<FORM METHOD=\"POST\" action=\"chat_send\" NAME=\"chatsendform\">\n");
+	wc_printf("<input type=\"hidden\" name=\"nonce\" value=\"%d\">\n", WC->nonce);
+	wc_printf("<INPUT TYPE=\"text\" SIZE=\"80\" MAXLENGTH=\"%d\" "
 		"NAME=\"send_this\">\n", SIZ-10);
-	wprintf("<br />");
-	wprintf("<INPUT TYPE=\"submit\" NAME=\"send_button\" VALUE=\"%s\">\n", _("Send"));
-	wprintf("<INPUT TYPE=\"submit\" NAME=\"help_button\" VALUE=\"%s\">\n", _("Help"));
-	wprintf("<INPUT TYPE=\"submit\" NAME=\"list_button\" VALUE=\"%s\">\n", _("List users"));
-	wprintf("<INPUT TYPE=\"submit\" NAME=\"exit_button\" VALUE=\"%s\">\n", _("Exit"));
-	wprintf("</FORM>\n");
+	wc_printf("<br />");
+	wc_printf("<INPUT TYPE=\"submit\" NAME=\"send_button\" VALUE=\"%s\">\n", _("Send"));
+	wc_printf("<INPUT TYPE=\"submit\" NAME=\"help_button\" VALUE=\"%s\">\n", _("Help"));
+	wc_printf("<INPUT TYPE=\"submit\" NAME=\"list_button\" VALUE=\"%s\">\n", _("List users"));
+	wc_printf("<INPUT TYPE=\"submit\" NAME=\"exit_button\" VALUE=\"%s\">\n", _("Exit"));
+	wc_printf("</FORM>\n");
 
-	wprintf("</BODY></HTML>\n");
+	wc_printf("</BODY></HTML>\n");
 	wDumpContent(0);
 }
 
@@ -520,10 +486,20 @@ void
 InitModule_PAGING
 (void)
 {
-	WebcitAddUrlHandler(HKEY("display_page"), display_page, 0);
-	WebcitAddUrlHandler(HKEY("page_user"), page_user, 0);
-	WebcitAddUrlHandler(HKEY("chat"), do_chat, 0);
-	WebcitAddUrlHandler(HKEY("chat_recv"), chat_recv, 0);
-	WebcitAddUrlHandler(HKEY("chat_send"), chat_send, 0);
-	WebcitAddUrlHandler(HKEY("ajax_send_instant_message"), ajax_send_instant_message, AJAX);
+	WebcitAddUrlHandler(HKEY("display_page"), "", 0, display_page, 0);
+	WebcitAddUrlHandler(HKEY("page_user"), "", 0, page_user, 0);
+	WebcitAddUrlHandler(HKEY("chat"), "", 0, do_chat, 0);
+	WebcitAddUrlHandler(HKEY("chat_recv"), "", 0, chat_recv, 0);
+	WebcitAddUrlHandler(HKEY("chat_send"), "", 0, chat_send, 0);
+	WebcitAddUrlHandler(HKEY("ajax_send_instant_message"), "", 0, ajax_send_instant_message, AJAX);
+	RegisterConditional(HKEY("COND:PAGE:WAITING"), 0, Conditional_PAGE_WAITING, CTX_NONE);
+}
+
+
+void 
+SessionDestroyModule_CHAT
+(wcsession *sess)
+{
+	if (sess->chat_sock > 0)
+		close(sess->chat_sock);
 }
