@@ -1,5 +1,5 @@
 /*
- * $Id: siteconfig.c 7280 2009-03-30 16:49:04Z ajc $
+ * $Id: siteconfig.c 7928 2009-10-20 12:52:58Z dothebart $
  *
  * Administrative screen for site-wide configuration
  */
@@ -301,13 +301,43 @@ int ConditionalServCfgSubst(StrBuf *Target, WCTemplputParams *TP)
 	else return 0;
 }
 
+void CfgZoneTempl(StrBuf *TemplBuffer, WCTemplputParams *TP)
+{
+	StrBuf *Zone = (StrBuf*) CTX;
+
+	SVPutBuf("ZONENAME", Zone, 1);
+}
+
 void 
 InitModule_SITECONFIG
 (void)
 {
-	WebcitAddUrlHandler(HKEY("siteconfig"), siteconfig, CTX_NONE);
+	WebcitAddUrlHandler(HKEY("siteconfig"), "", 0, siteconfig, CTX_NONE);
 
-	RegisterNamespace("SERV:CFG", 1, 2, tmplput_servcfg, CTX_NONE);
+	RegisterNamespace("SERV:CFG", 1, 2, tmplput_servcfg, NULL, CTX_NONE);
 	RegisterConditional(HKEY("COND:SERVCFG"), 3, ConditionalServCfg, CTX_NONE);
 	RegisterConditional(HKEY("COND:SERVCFG:SUBST"), 4, ConditionalServCfgSubst, CTX_NONE);
+	RegisterIterator("PREF:ZONE", 0, ZoneHash, NULL, CfgZoneTempl, NULL, CTX_PREF, CTX_NONE, IT_NOFLAG);
+}
+
+void 
+ServerStartModule_SITECONFIG
+(void)
+{
+	LoadZoneFiles();
+}
+
+void 
+ServerShutdownModule_SITECONFIG
+(void)
+{
+	DeleteHash(&ZoneHash);
+}
+
+
+void 
+SessionDestroyModule_SITECONFIG
+(wcsession *sess)
+{
+	DeleteHash(&sess->ServCfg);
 }
