@@ -1,5 +1,5 @@
 /*
- * $Id: wclib.js 7217 2009-03-16 19:50:49Z dothebart $
+ * $Id: wclib.js 8163 2009-12-29 13:37:42Z dothebart $
  * Copyright 2005 - 2009 The Citadel Team
  * Licensed under the GPL V3
  *
@@ -210,7 +210,8 @@ function switch_to_menu_buttons() {
   var roomlist = document.getElementById("roomlist");
   roomlist.className += " hidden";
 }
-function IconBarRoomList() {
+
+function GenericTreeRoomList(roomlist) {
   var currentExpanded = ctdlLocalPrefs.readPref("rooms_expanded");
   var curRoomName = "";
   if (document.getElementById("rmname")) {
@@ -218,7 +219,6 @@ function IconBarRoomList() {
   }
   currentDropTargets = new Array();
   var iconbar = document.getElementById("iconbar");
-  roomlist = document.getElementById("roomlist");
   var ul = document.createElement("ul");
   roomlist.appendChild(ul);
   // Add mailbox, because they are special
@@ -234,43 +234,57 @@ function IconBarRoomList() {
   mailboxLI.appendChild(mailboxUL);
   var mailboxRooms = GetMailboxRooms();
   for(var i=0; i<mailboxRooms.length; i++) {
-    var room = mailboxRooms[i];
-    currentDropTargets.push(addRoomToList(mailboxUL, room, curRoomName));
+	  var room = mailboxRooms[i];
+	  currentDropTargets.push(addRoomToList(mailboxUL, room, curRoomName));
   }
   if (currentExpanded != null && currentExpanded == _mailbox ) {
-    expandFloor(mailboxSPAN);
+	  expandFloor(mailboxSPAN);
   }
   for(var a=0; a<floors.length; a++) {
-    var floor = floors[a];
-    var floornum = floor[0];
-    var name = floor[1];
-    var floorLI = document.createElement("li");
-    ul.appendChild(floorLI);
-    var floorSPAN = document.createElement("span");
-    floorSPAN.appendChild(document.createTextNode(name));
-    $(floorSPAN).observe('click', expandFloorEvent);
-    floorLI.appendChild(floorSPAN);
-    floorLI.className = "floor";
-    var floorUL = document.createElement("ul");
-    floorLI.appendChild(floorUL);
-    var roomsForFloor = GetRoomsByFloorNum(floornum);
-    for(var b=0; b<roomsForFloor.length; b++) {
-      var room = roomsForFloor[b];
-      currentDropTargets.push(addRoomToList(floorUL, room, curRoomName));
-    }
-    if (currentExpanded != null && currentExpanded == name) {
-      expandFloor(floorSPAN);
+	  var floor = floors[a];
+	  var floornum = floor[0];
+    
+	  if (floornum != -1)
+	  {
+
+		  var name = floor[1];
+		  var floorLI = document.createElement("li");
+		  ul.appendChild(floorLI);
+		  var floorSPAN = document.createElement("span");
+		  floorSPAN.appendChild(document.createTextNode(name));
+		  $(floorSPAN).observe('click', expandFloorEvent);
+		  floorLI.appendChild(floorSPAN);
+		  floorLI.className = "floor";
+		  var floorUL = document.createElement("ul");
+		  floorLI.appendChild(floorUL);
+		  var roomsForFloor = GetRoomsByFloorNum(floornum);
+		  for(var b=0; b<roomsForFloor.length; b++) {
+			  var room = roomsForFloor[b];
+			  currentDropTargets.push(addRoomToList(floorUL, room, curRoomName));
+		  }
+		  if (currentExpanded != null && currentExpanded == name) {
+			  expandFloor(floorSPAN);
+		  }
     }
   }
 }
+function IconBarRoomList() {
+  roomlist = document.getElementById("roomlist");
+  GenericTreeRoomList(roomlist);
+}
+function KNRoomsRoomList() {
+  roomlist = document.getElementById("roomlist_knrooms");
+  GenericTreeRoomList(roomlist);
+}
 
 function addRoomToList(floorUL,room, roomToEmphasize) {
   var roomName = room[RN_ROOM_NAME];
   var flag = room[RN_ROOM_FLAG];
   var curView = room[RN_CUR_VIEW];
   var view = room[RN_DEF_VIEW];
+  var raflags = room[RN_RAFLAGS];
   var isMailBox = ((flag & QR_MAILBOX) == QR_MAILBOX);
-  var hasNewMsgs = ((curView & UA_HASNEWMSGS) == UA_HASNEWMSGS);
+  var hasNewMsgs = ((raflags & UA_HASNEWMSGS) == UA_HASNEWMSGS);
   var roomLI = document.createElement("li");
   var roomA = document.createElement("a");
   roomA.setAttribute("href","dotgoto?room="+roomName);
@@ -304,7 +318,7 @@ function addRoomToList(floorUL,room, roomToEmphasize) {
 }
 
 function roomListDropHandler(target, dropped) {
-  if (dropped.ctdlMsgId) {
+  if (dropped.getAttribute("citadel:msgid")) {
     var room = getTextContent(target);
     var msgIds = "";
     for(msgId in currentlyMarkedRows) { //defined in summaryview.js
@@ -509,8 +523,14 @@ function NotesResizeMouseMove(evt) {
 	divTop = parseInt(d.style.height);
 	divLeft = parseInt(d.style.width);
 
-	d.style.height = (divTop + y_increment) + 'px';
-	d.style.width = (divLeft + x_increment) + 'px';
+	newHeight = divTop + y_increment;
+	if (newHeight < 50) newHeight = 50;
+
+	newWidth = divLeft + x_increment;
+	if (newWidth < 50) newWidth = 50;
+
+	d.style.height = newHeight + 'px';
+	d.style.width = newWidth + 'px';
 
 	saved_x = x;
 	saved_y = y;
@@ -918,3 +938,19 @@ function resizeViewport() {
     global.style.width = newContentSize+"px";
   }
 }
+
+function RefreshSMTPqueueDisplay() {
+	new Ajax.Updater('smtpqueue_inner_div',
+	'display_smtpqueue_inner_div', { method: 'get',
+		parameters: Math.random() } );
+}
+
+function DeleteSMTPqueueMsg(msgnum1, msgnum2) {
+	new Ajax.Request(
+		'ajax_servcmd', {
+			method: 'post',
+			parameters: 'g_cmd=DELE ' + msgnum1 + ',' + msgnum2,
+			onComplete: RefreshSMTPqueueDisplay()
+		}
+	);
+}
