@@ -1,12 +1,6 @@
 /*
- * $Id: subst.c 7182 2009-03-03 22:39:20Z dothebart $
+ * $Id: subst.c 8186 2010-01-01 15:44:15Z dothebart $
  */
-/**
- * \defgroup Subst Variable substitution type stuff
- * \ingroup CitadelConfig
- */
-
-/*@{*/
 
 #include "sysdep.h"
 #include <sys/types.h>
@@ -20,7 +14,7 @@
 #include "webcit.h"
 #include "webserver.h"
 
-extern char *static_dirs[PATH_MAX];  /**< Disk representation */
+extern char *static_dirs[PATH_MAX];  /* Disk representation */
 
 HashList *WirelessTemplateCache;
 HashList *WirelessLocalTemplateCache;
@@ -31,6 +25,7 @@ HashList *GlobalNS;
 HashList *Iterators;
 HashList *Conditionals;
 HashList *SortHash;
+HashList *Defines;
 
 int DumpTemplateI18NStrings = 0;
 int LoadTemplates = 0;
@@ -38,6 +33,8 @@ int dbg_bactrace_template_errors = 0;
 WCTemplputParams NoCtx;
 StrBuf *I18nDump = NULL;
 
+const char EmptyStr[]="";
+
 #define SV_GETTEXT 1
 #define SV_CONDITIONAL 2
 #define SV_NEG_CONDITIONAL 3
@@ -47,15 +44,15 @@ StrBuf *I18nDump = NULL;
 
 
 /*
- * \brief Dynamic content for variable substitution in templates
+ * Dynamic content for variable substitution in templates
  */
 typedef struct _wcsubst {
 	ContextFilter Filter;
-	int wcs_type;			    /* which type of Substitution are we */
-	char wcs_key[32];		    /* copy of our hashkey for debugging */
-	StrBuf *wcs_value;		    /* if we're a string, keep it here */
-	long lvalue;                        /* type long? keep data here */
-	WCHandlerFunc wcs_function; /* funcion hook ???*/
+	int wcs_type;				/* which type of Substitution are we */
+	char wcs_key[32];			/* copy of our hashkey for debugging */
+	StrBuf *wcs_value;			/* if we're a string, keep it here */
+	long lvalue;				/* type long? keep data here */
+	WCHandlerFunc wcs_function;		/* funcion hook ???*/
 } wcsubst;
 
 
@@ -70,7 +67,7 @@ typedef struct _WCTemplate {
 
 typedef struct _HashHandler {
 	ContextFilter Filter;
-
+	WCPreevalFunc PreEvalFunc;
 	WCHandlerFunc HandlerFunc;
 }HashHandler;
 
@@ -107,6 +104,8 @@ const char *CtxNames[]  = {
 	"Context ROOMS",
 	"Context FLOORS",
 	"Context ITERATE",
+	"Context ICAL",
+	"Context DavNamespace",
 	"Context UNKNOWN"
 };
 
@@ -150,10 +149,14 @@ void LogTemplateError (StrBuf *Target, const char *Type, int ErrorPos, WCTemplpu
 		Err = (TP->Tokens!= NULL)? TP->Tokens->pName:"";
 		break;
 	case ERR_PARM1:
-		Err = (TP->Tokens!= NULL)? TP->Tokens->Params[0]->Start:"";
+		Err = ((TP->Tokens!= NULL) && 
+		       (TP->Tokens->nParameters > 0))? 
+			TP->Tokens->Params[0]->Start : "";
 		break;
 	case ERR_PARM2:
-		Err = (TP->Tokens!= NULL)? TP->Tokens->Params[1]->Start:"";
+		Err = ((TP->Tokens!= NULL) && 
+		       (TP->Tokens->nParameters > 1))? 
+			TP->Tokens->Params[1]->Start : "";
 		break;
 	}
 	if (TP->Tokens != NULL) 
@@ -177,6 +180,12 @@ void LogTemplateError (StrBuf *Target, const char *Type, int ErrorPos, WCTemplpu
 		return;
 */
 	WCC = WC;
+	if (WCC == NULL) {
+		FreeStrBuf(&Info);
+		FreeStrBuf(&Error);
+		return; 
+	}
+
 	Header = NewStrBuf();
 	if (TP->Tokens != NULL) 
 	{
@@ -208,7 +217,7 @@ void LogTemplateError (StrBuf *Target, const char *Type, int ErrorPos, WCTemplpu
 						      Error,
 						      eERROR), 1);
 */
-		WildFireSerializePayload(Header, WCC->HBuf, &WCC->nWildfireHeaders, NULL);
+		WildFireSerializePayload(Header, WCC->HBuf, &WCC->Hdr->nWildfireHeaders, NULL);
 	}
 	else
 	{
@@ -225,7 +234,7 @@ void LogTemplateError (StrBuf *Target, const char *Type, int ErrorPos, WCTemplpu
 			     ChrPtr(Error), 
 			     ChrPtr(TP->Tokens->FlatToken));
 		SerializeJson(Header, WildFireException(HKEY(__FILE__), __LINE__, Info, 1), 1);
-		WildFireSerializePayload(Header, WCC->HBuf, &WCC->nWildfireHeaders, NULL);
+		WildFireSerializePayload(Header, WCC->HBuf, &WCC->Hdr->nWildfireHeaders, NULL);
 	}
 	FreeStrBuf(&Header);
 	FreeStrBuf(&Info);
@@ -264,7 +273,7 @@ void LogError (StrBuf *Target, const char *Type, const char *Format, ...)
 						0,
 						Info,
 						1), 1);
-	WildFireSerializePayload(Header, WCC->HBuf, &WCC->nWildfireHeaders, NULL);
+	WildFireSerializePayload(Header, WCC->HBuf, &WCC->Hdr->nWildfireHeaders, NULL);
 	
 	FreeStrBuf(&Header);
 	FreeStrBuf(&Info);
@@ -281,16 +290,19 @@ void RegisterNS(const char *NSName,
 		int nMinArgs, 
 		int nMaxArgs, 
 		WCHandlerFunc HandlerFunc, 
+		WCPreevalFunc PreevalFunc,
 		int ContextRequired)
 {
 	HashHandler *NewHandler;
 	
 	NewHandler = (HashHandler*) malloc(sizeof(HashHandler));
+	memset(NewHandler, 0, sizeof(HashHandler));
 	NewHandler->Filter.nMinArgs = nMinArgs;
 	NewHandler->Filter.nMaxArgs = nMaxArgs;
 	NewHandler->Filter.ContextType = ContextRequired;
 	NewHandler->Filter.ControlContextType = CTX_NONE;
 
+	NewHandler->PreEvalFunc = PreevalFunc;
 	NewHandler->HandlerFunc = HandlerFunc;	
 	Put(GlobalNS, NSName, len, NewHandler, NULL);
 }
@@ -305,6 +317,7 @@ void RegisterControlNS(const char *NSName,
 	HashHandler *NewHandler;
 	
 	NewHandler = (HashHandler*) malloc(sizeof(HashHandler));
+	memset(NewHandler, 0, sizeof(HashHandler));
 	NewHandler->Filter.nMinArgs = nMinArgs;
 	NewHandler->Filter.nMaxArgs = nMaxArgs;
 	NewHandler->Filter.ContextType = CTX_NONE;
@@ -388,12 +401,13 @@ void FreeWCTemplate(void *vFreeMe)
 }
 
 
-/**
- * \brief debugging function to print array to log
+/*
+ * debugging function to print array to log
  */
 void VarPrintTransition(void *vVar1, void *vVar2, int odd){}
-/**
- * \brief debugging function to print array to log
+
+/*
+ * debugging function to print array to log
  */
 void VarPrintEntry(const char *Key, void *vSubst, int odd)
 {
@@ -418,8 +432,8 @@ void VarPrintEntry(const char *Key, void *vSubst, int odd)
 
 
 
-/**
- * \brief Clear out the list of substitution variables local to this session
+/*
+ * Clear out the list of substitution variables local to this session
  */
 void clear_substs(wcsession *wc) {
 
@@ -428,8 +442,8 @@ void clear_substs(wcsession *wc) {
 	}
 }
 
-/**
- * \brief Clear out the list of substitution variables local to this session
+/*
+ * Clear out the list of substitution variables local to this session
  */
 void clear_local_substs(void) {
 	clear_substs (WC);
@@ -489,8 +503,8 @@ void FlushPayload(wcsubst *ptr, int reusestrbuf, int type)
 }
 
 
-/**
- * \brief destructor; kill one entry.
+/*
+ * destructor; kill one entry.
  */
 void deletevar(void *data)
 {
@@ -528,12 +542,12 @@ wcsubst *NewSubstVar(const char *keyname, int keylen, int type)
 }
 
 
-/**
- * \brief Add a substitution variable (local to this session) (strlen version...)
- * \param keyname the replacementstring to substitute
- * \param keytype the kind of the key
- * \param format the format string ala printf
- * \param ... the arguments to substitute in the formatstring
+/*
+ * Add a substitution variable (local to this session) (strlen version...)
+ * keyname the replacementstring to substitute
+ * keytype the kind of the key
+ * format the format string ala printf
+ * ... the arguments to substitute in the formatstring
  */
 void SVPRINTF(char *keyname, int keytype, const char *format,...)
 {
@@ -544,7 +558,7 @@ void SVPRINTF(char *keyname, int keytype, const char *format,...)
 	wcsession *WCC = WC;
 	
 	keylen = strlen(keyname);
-	/**
+	/*
 	 * First look if we're doing a replacement of
 	 * an existing key
 	 */
@@ -559,18 +573,18 @@ void SVPRINTF(char *keyname, int keytype, const char *format,...)
 		ptr = NewSubstVar(keyname, keylen, keytype);
 	}
 
-	/** Format the string */
+	/* Format the string */
 	va_start(arg_ptr, format);
 	StrBufVAppendPrintf(ptr->wcs_value, format, arg_ptr);
 	va_end(arg_ptr);
 }
 
-/**
- * \brief Add a substitution variable (local to this session)
- * \param keyname the replacementstring to substitute
- * \param keytype the kind of the key
- * \param format the format string ala printf
- * \param ... the arguments to substitute in the formatstring
+/*
+ * Add a substitution variable (local to this session)
+ * keyname the replacementstring to substitute
+ * keytype the kind of the key
+ * format the format string ala printf
+ * ... the arguments to substitute in the formatstring
  */
 void svprintf(char *keyname, size_t keylen, int keytype, const char *format,...)
 {
@@ -579,7 +593,7 @@ void svprintf(char *keyname, size_t keylen, int keytype, const char *format,...)
 	wcsubst *ptr = NULL;
 	wcsession *WCC = WC;
 		
-	/**
+	/*
 	 * First look if we're doing a replacement of
 	 * an existing key
 	 */
@@ -600,12 +614,12 @@ void svprintf(char *keyname, size_t keylen, int keytype, const char *format,...)
 	va_end(arg_ptr);
 }
 
-/**
- * \brief Add a substitution variable (local to this session)
- * \param keyname the replacementstring to substitute
- * \param keytype the kind of the key
- * \param format the format string ala printf
- * \param ... the arguments to substitute in the formatstring
+/*
+ * Add a substitution variable (local to this session)
+ * keyname the replacementstring to substitute
+ * keytype the kind of the key
+ * format the format string ala printf
+ * ... the arguments to substitute in the formatstring
  */
 void SVPut(char *keyname, size_t keylen, int keytype, char *Data)
 {
@@ -614,7 +628,7 @@ void SVPut(char *keyname, size_t keylen, int keytype, char *Data)
 	wcsession *WCC = WC;
 
 	
-	/**
+	/*
 	 * First look if we're doing a replacement of
 	 * an existing key
 	 */
@@ -756,7 +770,7 @@ void GetTemplateTokenString(StrBuf *Target,
 	StrBuf *Buf;
 	WCTemplputParams SubTP;
 
-	if (TP->Tokens->nParameters < N) {
+	if (N >= TP->Tokens->nParameters) {
 		LogTemplateError(Target, 
 				 "TokenParameter", N, TP, 
 				 "invalid token %d. this shouldn't have come till here.\n", N);
@@ -767,16 +781,33 @@ void GetTemplateTokenString(StrBuf *Target,
 
 	switch (TP->Tokens->Params[N]->Type) {
 
+	case TYPE_INTDEFINE:
 	case TYPE_STR:
 		*Value = TP->Tokens->Params[N]->Start;
 		*len = TP->Tokens->Params[N]->len;
 		break;
 	case TYPE_BSTR:
+		if (TP->Tokens->Params[N]->len == 0) {
+			LogTemplateError(Target, 
+					 "TokenParameter", N, TP, 
+					 "Requesting parameter %d; of type BSTR, empty lookup string not admitted.", N);
+			*len = 0;
+			*Value = EmptyStr;
+			break;
+		}
 		Buf = (StrBuf*) SBstr(TKEY(N));
 		*Value = ChrPtr(Buf);
 		*len = StrLength(Buf);
 		break;
 	case TYPE_PREFSTR:
+		if (TP->Tokens->Params[N]->len == 0) {
+			LogTemplateError(Target, 
+					 "TokenParameter", N, TP, 
+					 "Requesting parameter %d; of type PREFSTR, empty lookup string not admitted.", N);
+			*len = 0;
+			*Value = EmptyStr;
+			break;
+		}
 		get_PREFERENCE(TKEY(N), &Buf);
 		*Value = ChrPtr(Buf);
 		*len = StrLength(Buf);
@@ -796,6 +827,15 @@ void GetTemplateTokenString(StrBuf *Target,
 		*len = strlen(*Value);
 		break;
 	case TYPE_SUBTEMPLATE:
+		if (TP->Tokens->Params[N]->len == 0) {
+			LogTemplateError(Target, 
+					 "TokenParameter", N, TP, 
+					 "Requesting parameter %d; of type SUBTEMPLATE, empty lookup string not admitted.", N);
+			*len = 0;
+			*Value = EmptyStr;
+			break;
+		}
+
 		memset(&SubTP, 0, sizeof(WCTemplputParams *));
 		SubTP.Context = TP->Context;
 		SubTP.Filter.ContextType = TP->Filter.ContextType;
@@ -818,7 +858,7 @@ void GetTemplateTokenString(StrBuf *Target,
 long GetTemplateTokenNumber(StrBuf *Target, WCTemplputParams *TP, int N, long dflt)
 {
 	long Ret;
-	if (TP->Tokens->nParameters < N) {
+	if (N >= TP->Tokens->nParameters) {
 		LogTemplateError(Target, 
 				 "TokenParameter", N, TP, 
 				 "invalid token %d. this shouldn't have come till here.\n", N);
@@ -832,18 +872,37 @@ long GetTemplateTokenNumber(StrBuf *Target, WCTemplputParams *TP, int N, long df
 		return atol(TP->Tokens->Params[N]->Start);
 		break;
 	case TYPE_BSTR:
+		if (TP->Tokens->Params[N]->len == 0) {
+			LogTemplateError(Target, 
+					 "TokenParameter", N, TP, 
+					 "Requesting parameter %d; of type BSTR, empty lookup string not admitted.", N);
+			return 0;
+		}
 		return  LBstr(TKEY(N));
 		break;
 	case TYPE_PREFSTR:
 		LogTemplateError(Target, 
 				 "TokenParameter", N, TP, 
 				 "requesting a prefstring in param %d want a number", N);
+		if (TP->Tokens->Params[N]->len == 0) {
+			LogTemplateError(Target, 
+					 "TokenParameter", N, TP, 
+					 "Requesting parameter %d; of type PREFSTR, empty lookup string not admitted.", N);
+			return 0;
+		}
 		if (get_PREF_LONG(TKEY(N), &Ret, dflt))
 			return Ret;
-		return 0;		
+		return 0;
+	case TYPE_INTDEFINE:
 	case TYPE_LONG:
 		return TP->Tokens->Params[N]->lvalue;
 	case TYPE_PREFINT:
+		if (TP->Tokens->Params[N]->len == 0) {
+			LogTemplateError(Target, 
+					 "TokenParameter", N, TP, 
+					 "Requesting parameter %d; of type PREFINT, empty lookup string not admitted.", N);
+			return 0;
+		}
 		if (get_PREF_LONG(TKEY(N), &Ret, dflt))
 			return Ret;
 		return 0;		
@@ -1019,6 +1078,9 @@ void StrBufAppendTemplate(StrBuf *Target,
 	case 'J':
 		StrECMAEscAppend(Target, Source, NULL);
 	  break;
+	case 'K':
+		StrHtmlEcmaEscAppend(Target, Source, NULL, 0, 0);
+	  break;
 	case 'U':
 		StrBufUrlescAppend(Target, Source, NULL);
 		break;
@@ -1034,6 +1096,7 @@ void PutNewToken(WCTemplate *Template, WCTemplateToken *NewToken)
 		if (Template->TokenSpace <= 0) {
 			Template->Tokens = (WCTemplateToken**)malloc(
 				sizeof(WCTemplateToken*) * 10);
+			memset(Template->Tokens, 0, sizeof(WCTemplateToken*));
 			Template->TokenSpace = 10;
 		}
 		else {
@@ -1051,14 +1114,21 @@ void PutNewToken(WCTemplate *Template, WCTemplateToken *NewToken)
 	Template->Tokens[(Template->nTokensUsed)++] = NewToken;
 }
 
-TemplateParam *GetNextParameter(StrBuf *Buf, const char **pCh, const char *pe, WCTemplateToken *Tokens, WCTemplate *pTmpl)
+TemplateParam *GetNextParameter(StrBuf *Buf, 
+				const char **pCh, 
+				const char *pe, 
+				WCTemplateToken *Tokens, 
+				WCTemplate *pTmpl, 
+				WCTemplputParams *TP)
 {
 	const char *pch = *pCh;
 	const char *pchs, *pche;
-	TemplateParam *Parm = (TemplateParam *) malloc(sizeof(TemplateParam));
+	TemplateParam *Parm;
 	char quote = '\0';
 	int ParamBrace = 0;
 
+	Parm = (TemplateParam *) malloc(sizeof(TemplateParam));
+	memset(Parm, 0, sizeof(TemplateParam));
 	Parm->Type = TYPE_STR;
 
 	/* Skip leading whitespaces */
@@ -1083,6 +1153,10 @@ TemplateParam *GetNextParameter(StrBuf *Buf, const char **pCh, const char *pe, W
 			ParamBrace = 1;
 		}
 	}
+	else if (*pch == '#') {
+		Parm->Type = TYPE_INTDEFINE;
+		pch ++;
+	}
 	else if (*pch == '_') {
 		Parm->Type = TYPE_GETTEXT;
 		pch ++;
@@ -1187,6 +1261,25 @@ TemplateParam *GetNextParameter(StrBuf *Buf, const char **pCh, const char *pe, W
 	if (DumpTemplateI18NStrings && (Parm->Type == TYPE_GETTEXT)) {
 		StrBufAppendPrintf(I18nDump, "_(\"%s\");\n", Parm->Start);
 	}
+	if (Parm->Type == TYPE_INTDEFINE)
+	{
+		void *vPVal;
+
+		if (GetHash(Defines, Parm->Start, Parm->len, &vPVal) &&
+		    (vPVal != NULL))
+		{
+			long *PVal;
+			PVal = (long*) vPVal;
+		
+			Parm->lvalue = *PVal;
+		}
+		else 
+		{
+			LogTemplateError(NULL, "Define", ERR_PARM1, TP,
+					 "%s isn't known!!",
+					 Parm->Start);
+		}
+	}
 	*pCh = pch;
 	return Parm;
 }
@@ -1201,9 +1294,11 @@ WCTemplateToken *NewTemplateSubstitute(StrBuf *Buf,
 	void *vVar;
 	const char *pch;
 	TemplateParam *Param;
-	WCTemplateToken *NewToken = (WCTemplateToken*)malloc(sizeof(WCTemplateToken));
+	WCTemplateToken *NewToken;
 	WCTemplputParams TP;
 
+	NewToken = (WCTemplateToken*)malloc(sizeof(WCTemplateToken));
+	memset(NewToken, 0, sizeof(WCTemplateToken));
 	TP.Tokens = NewToken;
 	NewToken->FileName = pTmpl->FileName; /* to print meaningfull log messages... */
 	NewToken->Flags = 0;
@@ -1215,7 +1310,8 @@ WCTemplateToken *NewTemplateSubstitute(StrBuf *Buf,
 	NewToken->NameEnd = NewToken->TokenEnd - 2;
 	NewToken->PreEval = NULL;
 	NewToken->FlatToken = NewStrBufPlain(pTmplStart + 2, pTmplEnd - pTmplStart - 2);
-	
+	StrBufShrinkToFit(NewToken->FlatToken, 1);
+
 	StrBufPeek(Buf, pTmplStart, + 1, '\0');
 	StrBufPeek(Buf, pTmplEnd, -1, '\0');
 	pch = NewToken->pName = pTmplStart + 2;
@@ -1234,7 +1330,7 @@ WCTemplateToken *NewTemplateSubstitute(StrBuf *Buf,
 					"Warning, Non welformed Token; missing right parenthesis");
 			}
 			while (pch < pTmplEnd - 1) {
-				Param = GetNextParameter(Buf, &pch, pTmplEnd - 1, NewToken, pTmpl);
+				Param = GetNextParameter(Buf, &pch, pTmplEnd - 1, NewToken, pTmpl, &TP);
 				if (Param != NULL) {
 					NewToken->HaveParameters = 1;
 					if (NewToken->nParameters > MAXPARAM) {
@@ -1287,6 +1383,8 @@ WCTemplateToken *NewTemplateSubstitute(StrBuf *Buf,
 			else {
 				NewToken->PreEval = Handler;
 				NewToken->Flags = SV_PREEVALUATED;		
+				if (Handler->PreEvalFunc != NULL)
+					Handler->PreEvalFunc(NewToken);
 			}
 		}
 		break;
@@ -1371,9 +1469,12 @@ WCTemplateToken *NewTemplateSubstitute(StrBuf *Buf,
 void *prepare_template(StrBuf *filename, StrBuf *Key, HashList *PutThere)
 {
 	WCTemplate *NewTemplate;
+
 	NewTemplate = (WCTemplate *) malloc(sizeof(WCTemplate));
+	memset(NewTemplate, 0, sizeof(WCTemplate));
 	NewTemplate->Data = NULL;
 	NewTemplate->FileName = NewStrBufDup(filename);
+	StrBufShrinkToFit(NewTemplate->FileName, 1);
 	NewTemplate->nTokensUsed = 0;
 	NewTemplate->TokenSpace = 0;
 	NewTemplate->Tokens = NULL;
@@ -1413,6 +1514,7 @@ void *load_template(StrBuf *filename, StrBuf *Key, HashList *PutThere)
 	}
 
 	NewTemplate = (WCTemplate *) malloc(sizeof(WCTemplate));
+	memset(NewTemplate, 0, sizeof(WCTemplate));
 	NewTemplate->Data = NewStrBufPlain(NULL, statbuf.st_size);
 	NewTemplate->FileName = NewStrBufDup(filename);
 	NewTemplate->nTokensUsed = 0;
@@ -1433,6 +1535,8 @@ void *load_template(StrBuf *filename, StrBuf *Key, HashList *PutThere)
 	close(fd);
 
 	Line = 0;
+	StrBufShrinkToFit(NewTemplate->Data, 1);
+	StrBufShrinkToFit(NewTemplate->MimeType, 1);
 	pS = pch = ChrPtr(NewTemplate->Data);
 	pE = pS + StrLength(NewTemplate->Data);
 	while (pch < pE) {
@@ -1443,7 +1547,11 @@ void *load_template(StrBuf *filename, StrBuf *Key, HashList *PutThere)
 		/** Find one <? > */
 		pos = (-1);
 		for (; pch < pE; pch ++) {
-			if ((*pch=='<')&&(*(pch + 1)=='?'))
+			if ((*pch=='<')&&(*(pch + 1)=='?') &&
+			    !((pch == pS) && /* we must ommit a <?xml */
+			      (*(pch + 2) == 'x') && 
+			      (*(pch + 3) == 'm') && 
+			      (*(pch + 4) == 'l')))			     
 				break;
 			if (*pch=='\n') Line ++;
 		}
@@ -1737,7 +1845,7 @@ const StrBuf *DoTemplate(const char *templatename, long len, StrBuf *Target, WCT
 		TP = &LocalTP;
 	}
 
-	if (WC->is_mobile) {
+	if (WC->is_mobile > 0) {
 		Static = WirelessTemplateCache;
 		StaticLocal = WirelessLocalTemplateCache;
 	}
@@ -1795,7 +1903,10 @@ void RegisterITERATOR(const char *Name, long len,
 		      int XPectContextType, 
 		      int Flags)
 {
-	HashIterator *It = (HashIterator*)malloc(sizeof(HashIterator));
+	HashIterator *It;
+
+	It = (HashIterator*)malloc(sizeof(HashIterator));
+	memset(It, 0, sizeof(HashIterator));
 	It->StaticList = StaticList;
 	It->AdditionalParams = AdditionalParams;
 	It->GetHash = GetHash;
@@ -1816,9 +1927,27 @@ typedef struct _iteratestruct {
 	int LastN;
 	}IterateStruct; 
 
+int preeval_iterate(WCTemplateToken *Token)
+{
+	WCTemplputParams TPP;
+	WCTemplputParams *TP;
+	void *vIt;
+
+	memset(&TPP, 0, sizeof(WCTemplputParams));
+	TP = &TPP;
+	TP->Tokens = Token;
+	if (!GetHash(Iterators, TKEY(0), &vIt)) {
+		LogTemplateError(
+			NULL, "Iterator", ERR_NAME, TP,
+			"not found");
+		return 0;
+	}
+	Token->Preeval2 = vIt;
+	return 1;
+}
+
 void tmpl_iterate_subtmpl(StrBuf *Target, WCTemplputParams *TP)
 {
-	void *vIt;
 	HashIterator *It;
 	HashList *List;
 	HashPos  *it;
@@ -1839,14 +1968,13 @@ void tmpl_iterate_subtmpl(StrBuf *Target, WCTemplputParams *TP)
 	memset(&Status, 0, sizeof(IterateStruct));
 	memcpy (&SubTP, &TP, sizeof(WCTemplputParams));
 	
-	if (!GetHash(Iterators, TKEY(0), &vIt)) {
+	It = (HashIterator*) TP->Tokens->Preeval2;
+	if (It == NULL) {
 		LogTemplateError(
 			Target, "Iterator", ERR_PARM1, TP, "Unknown!");
 		return;
 	}
 
-	It = (HashIterator*) vIt;
-
 	if (TP->Tokens->nParameters < It->AdditionalParams + 2) {
 		LogTemplateError(                               
 			Target, "Iterator", ERR_PARM1, TP,
@@ -1912,7 +2040,7 @@ void tmpl_iterate_subtmpl(StrBuf *Target, WCTemplputParams *TP)
 	while (GetNextHashPos(List, it, &Status.KeyLen, &Status.Key, &vContext)) {
 		if ((Status.n >= StartAt) && (Status.n <= StopAt)) {
 			if (DetectGroupChange && Status.n > 0) {
-				Status.GroupChange = (SortBy->GroupChange(vContext, vLastContext))? 1:0;
+				Status.GroupChange = SortBy->GroupChange(vContext, vLastContext);
 			}
 			Status.LastN = (Status.n + 1) == nMembersUsed;
 			SubTP.Context = vContext;
@@ -1937,7 +2065,10 @@ void tmpl_iterate_subtmpl(StrBuf *Target, WCTemplputParams *TP)
 int conditional_ITERATE_ISGROUPCHANGE(StrBuf *Target, WCTemplputParams *TP)
 {
 	IterateStruct *Ctx = CCTX;
-	return Ctx->GroupChange;
+	if (TP->Tokens->nParameters < 3)
+		return 	Ctx->GroupChange;
+
+	return TP->Tokens->Params[2]->lvalue == Ctx->GroupChange;
 }
 
 void tmplput_ITERATE_ODDEVEN(StrBuf *Target, WCTemplputParams *TP)
@@ -1964,6 +2095,12 @@ void tmplput_ITERATE_LASTN(StrBuf *Target, WCTemplputParams *TP)
 	StrBufAppendPrintf(Target, "%d", Ctx->n);
 }
 
+int conditional_ITERATE_FIRSTN(StrBuf *Target, WCTemplputParams *TP)
+{
+	IterateStruct *Ctx = CCTX;
+	return Ctx->n == 0;
+}
+
 int conditional_ITERATE_LASTN(StrBuf *Target, WCTemplputParams *TP)
 {
 	IterateStruct *Ctx = CCTX;
@@ -2037,7 +2174,10 @@ void RegisterConditional(const char *Name, long len,
 			 WCConditionalFunc CondF, 
 			 int ContextRequired)
 {
-	ConditionalStruct *Cond = (ConditionalStruct*)malloc(sizeof(ConditionalStruct));
+	ConditionalStruct *Cond;
+
+	Cond = (ConditionalStruct*)malloc(sizeof(ConditionalStruct));
+	memset(Cond, 0, sizeof(ConditionalStruct));
 	Cond->PlainName = Name;
 	Cond->Filter.nMaxArgs = nParams;
 	Cond->Filter.nMinArgs = nParams;
@@ -2052,7 +2192,10 @@ void RegisterControlConditional(const char *Name, long len,
 				WCConditionalFunc CondF, 
 				int ControlContextRequired)
 {
-	ConditionalStruct *Cond = (ConditionalStruct*)malloc(sizeof(ConditionalStruct));
+	ConditionalStruct *Cond;
+
+	Cond = (ConditionalStruct*)malloc(sizeof(ConditionalStruct));
+	memset(Cond, 0, sizeof(ConditionalStruct));
 	Cond->PlainName = Name;
 	Cond->Filter.nMaxArgs = nParams;
 	Cond->Filter.nMinArgs = nParams;
@@ -2062,6 +2205,18 @@ void RegisterControlConditional(const char *Name, long len,
 	Put(Conditionals, Name, len, Cond, NULL);
 }
 
+void RegisterTokenParamDefine(const char *Name, long len, 
+			      long Value)
+{
+	long *PVal;
+
+	PVal = (long*)malloc(sizeof(long));
+	*PVal = Value;
+	Put(Defines, Name, len, PVal, NULL);
+}
+
+HashList *Defines;
+
 /*-----------------------------------------------------------------------------
  *                      Context Strings
  */
@@ -2087,7 +2242,7 @@ void tmpl_do_boxed(StrBuf *Target, WCTemplputParams *TP)
 {
 	WCTemplputParams SubTP;
 
-	StrBuf *Headline;
+	StrBuf *Headline = NULL;
 	if (TP->Tokens->nParameters == 2) {
 		if (TP->Tokens->Params[1]->Type == TYPE_STR) {
 			Headline = NewStrBuf();
@@ -2104,7 +2259,8 @@ void tmpl_do_boxed(StrBuf *Target, WCTemplputParams *TP)
 			Headline = NewStrBufPlain(Ch, len);
 		}
 	}
-       memcpy (&SubTP, TP, sizeof(WCTemplputParams));
+	/* else TODO error? logging? */
+	memcpy (&SubTP, TP, sizeof(WCTemplputParams));
 	SubTP.Context = Headline;
 	SubTP.Filter.ContextType = CTX_STRBUF;
 	DoTemplate(HKEY("beginbox"), Target, &SubTP);
@@ -2124,6 +2280,7 @@ void tmpl_do_tabbed(StrBuf *Target, WCTemplputParams *TP)
 
 	nTabs = ntabs = TP->Tokens->nParameters / 2;
 	TabNames = (StrBuf **) malloc(ntabs * sizeof(StrBuf*));
+	memset(TabNames, 0, ntabs * sizeof(StrBuf*));
 
 	for (i = 0; i < ntabs; i++) {
 		if ((TP->Tokens->Params[i * 2]->Type == TYPE_STR) &&
@@ -2168,7 +2325,10 @@ void RegisterSortFunc(const char *name, long len,
 		      CompareFunc GroupChange, 
 		      long ContextType)
 {
-	SortStruct *NewSort = (SortStruct*) malloc(sizeof(SortStruct));
+	SortStruct *NewSort;
+
+	NewSort = (SortStruct*) malloc(sizeof(SortStruct));
+	memset(NewSort, 0, sizeof(SortStruct));
 	NewSort->Name = NewStrBufPlain(name, len);
 	if (prepend != NULL)
 		NewSort->PrefPrepend = NewStrBufPlain(prepend, preplen);
@@ -2178,6 +2338,11 @@ void RegisterSortFunc(const char *name, long len,
 	NewSort->Reverse = Reverse;
 	NewSort->GroupChange = GroupChange;
 	NewSort->ContextType = ContextType;
+	if (ContextType == CTX_NONE) {
+		lprintf(1, "sorting requires a context. CTX_NONE won't make it.\n");
+		exit(1);
+	}
+		
 	Put(SortHash, name, len, NewSort, DestroySortStruct);
 }
 
@@ -2230,6 +2395,9 @@ CompareFunc RetrieveSort(WCTemplputParams *TP,
 	}
 	SortBy = (SortStruct*)vSortBy;
 
+	if (SortBy->ContextType != TP->Filter.ContextType)
+		return NULL;
+
 	/** Ok, its us, lets see in which direction we should sort... */
 	if (havebstr("SortOrder")) {
 		SortOrder = LBSTR("SortOrder");
@@ -2503,19 +2671,25 @@ int ConditionalLongVector(StrBuf *Target, WCTemplputParams *TP)
 	return 0;
 }
 
+
+void tmplput_CURRENT_FILE(StrBuf *Target, WCTemplputParams *TP)
+{
+	StrBufAppendTemplate(Target, TP, TP->Tokens->FileName, 0);
+}
+
 void 
 InitModule_SUBST
 (void)
 {
 	memset(&NoCtx, 0, sizeof(WCTemplputParams));
-	RegisterNamespace("SORT:ICON", 1, 2, tmplput_SORT_ICON, CTX_NONE);
-	RegisterNamespace("SORT:ORDER", 1, 2, tmplput_SORT_ORDER, CTX_NONE);
-	RegisterNamespace("SORT:NEXT", 1, 2, tmplput_SORT_NEXT, CTX_NONE);
-	RegisterNamespace("CONTEXTSTR", 0, 1, tmplput_ContextString, CTX_STRBUF);
-	RegisterNamespace("ITERATE", 2, 100, tmpl_iterate_subtmpl, CTX_NONE);
-	RegisterNamespace("DOBOXED", 1, 2, tmpl_do_boxed, CTX_NONE);
-	RegisterNamespace("DOTABBED", 2, 100, tmpl_do_tabbed, CTX_NONE);
-	RegisterNamespace("LONGVECTOR", 1, 1, tmplput_long_vector, CTX_LONGVECTOR);
+	RegisterNamespace("SORT:ICON", 1, 2, tmplput_SORT_ICON, NULL, CTX_NONE);
+	RegisterNamespace("SORT:ORDER", 1, 2, tmplput_SORT_ORDER, NULL, CTX_NONE);
+	RegisterNamespace("SORT:NEXT", 1, 2, tmplput_SORT_NEXT, NULL, CTX_NONE);
+	RegisterNamespace("CONTEXTSTR", 0, 1, tmplput_ContextString, NULL, CTX_STRBUF);
+	RegisterNamespace("ITERATE", 2, 100, tmpl_iterate_subtmpl, preeval_iterate, CTX_NONE);
+	RegisterNamespace("DOBOXED", 1, 2, tmpl_do_boxed, NULL, CTX_NONE);
+	RegisterNamespace("DOTABBED", 2, 100, tmpl_do_tabbed, NULL, CTX_NONE);
+	RegisterNamespace("LONGVECTOR", 1, 1, tmplput_long_vector, NULL, CTX_LONGVECTOR);
 	RegisterConditional(HKEY("COND:SUBST"), 3, ConditionalVar, CTX_NONE);
 	RegisterConditional(HKEY("COND:CONTEXTSTR"), 3, ConditionalContextStr, CTX_STRBUF);
 	RegisterConditional(HKEY("COND:LONGVECTOR"), 4, ConditionalLongVector, CTX_LONGVECTOR);
@@ -2526,9 +2700,82 @@ InitModule_SUBST
 	RegisterControlConditional(HKEY("COND:ITERATE:LASTN"), 2, 
 				   conditional_ITERATE_LASTN, 
 				   CTX_ITERATE);
+	RegisterControlConditional(HKEY("COND:ITERATE:FIRSTN"), 2, 
+				   conditional_ITERATE_FIRSTN, 
+				   CTX_ITERATE);
+
 	RegisterControlNS(HKEY("ITERATE:ODDEVEN"), 0, 0, tmplput_ITERATE_ODDEVEN, CTX_ITERATE);
 	RegisterControlNS(HKEY("ITERATE:KEY"), 0, 0, tmplput_ITERATE_KEY, CTX_ITERATE);
 	RegisterControlNS(HKEY("ITERATE:N"), 0, 0, tmplput_ITERATE_LASTN, CTX_ITERATE);
+	RegisterNamespace("CURRENTFILE", 0, 1, tmplput_CURRENT_FILE, NULL, CTX_NONE);
+
+
 }
 
-/*@}*/
+void
+ServerStartModule_SUBST
+(void)
+{
+	WirelessTemplateCache = NewHash(1, NULL);
+	WirelessLocalTemplateCache = NewHash(1, NULL);
+	LocalTemplateCache = NewHash(1, NULL);
+	TemplateCache = NewHash(1, NULL);
+
+	GlobalNS = NewHash(1, NULL);
+	Iterators = NewHash(1, NULL);
+	Conditionals = NewHash(1, NULL);
+	SortHash = NewHash(1, NULL);
+	Defines = NewHash(1, NULL);
+}
+
+void
+FinalizeModule_SUBST
+(void)
+{
+
+}
+
+void 
+ServerShutdownModule_SUBST
+(void)
+{
+	DeleteHash(&WirelessTemplateCache);
+	DeleteHash(&WirelessLocalTemplateCache);
+	DeleteHash(&TemplateCache);
+	DeleteHash(&LocalTemplateCache);
+
+	DeleteHash(&GlobalNS);
+	DeleteHash(&Iterators);
+	DeleteHash(&Conditionals);
+	DeleteHash(&SortHash);
+	DeleteHash(&Defines);
+}
+
+
+void
+SessionNewModule_SUBST
+(wcsession *sess)
+{
+
+}
+
+void
+SessionAttachModule_SUBST
+(wcsession *sess)
+{
+	sess->vars = NewHash(1,NULL);
+}
+
+void
+SessionDetachModule_SUBST
+(wcsession *sess)
+{
+	DeleteHash(&sess->vars);
+}
+
+void 
+SessionDestroyModule_SUBST  
+(wcsession *sess)
+{
+
+}
