@@ -1,10 +1,11 @@
 /*
- * $Id: summary.c 7087 2009-02-10 18:50:50Z dothebart $
+ * $Id: summary.c 7939 2009-10-23 12:51:53Z dothebart $
  *
  * Displays the "Summary Page"
  */
 
 #include "webcit.h"
+#include "calendar.h"
 
 /*
  * Display today's date in a friendly format
@@ -18,7 +19,7 @@ void output_date(void) {
 	localtime_r(&now, &tm);
 
 	wc_strftime(buf, 32, "%A, %x", &tm);
-	wprintf("%s", buf);
+	wc_printf("%s", buf);
 }
 
 
@@ -30,7 +31,7 @@ void output_date(void) {
 void dummy_section(void) {
 	svput("BOXTITLE", WCS_STRING, "(dummy&nbsp;section)");
 	do_template("beginboxx", NULL);
-	wprintf(_("(nothing)"));
+	wc_printf(_("(nothing)"));
 	do_template("endbox", NULL);
 }
 
@@ -49,7 +50,7 @@ void new_messages_section(void) {
 	number_of_rooms_to_check = num_tokens(rooms_to_check, '|');
 	if (number_of_rooms_to_check == 0) return;
 
-	wprintf("<table border=0 width=100%%>\n");
+	wc_printf("<table border=0 width=100%%>\n");
 	for (i=0; i<number_of_rooms_to_check; ++i) {
 		extract_token(room, rooms_to_check, i, '|', sizeof room);
 
@@ -57,17 +58,17 @@ void new_messages_section(void) {
 		serv_getln(buf, sizeof buf);
 		if (buf[0] == '2') {
 			extract_token(room, &buf[4], 0, '|', sizeof room);
-			wprintf("<tr><td><a href=\"dotgoto?room=");
+			wc_printf("<tr><td><a href=\"dotgoto?room=");
 			urlescputs(room);
-			wprintf("\">");
+			wc_printf("\">");
 			escputs(room);
-			wprintf("</a></td><td>%d/%d</td></tr>\n",
+			wc_printf("</a></td><td>%d/%d</td></tr>\n",
 				extract_int(&buf[4], 1),
 				extract_int(&buf[4], 2)
 			);
 		}
 	}
-	wprintf("</table>\n");
+	wc_printf("</table>\n");
 
 }
 
@@ -84,6 +85,12 @@ void tasks_section(void) {
 	message_summary *Msg;
 	wcsession *WCC = WC;
 	StrBuf *Buf;
+	SharedMessageStatus Stat;
+
+	memset(&Stat, 0, sizeof(SharedMessageStatus));
+	Stat.maxload = 10000;
+	Stat.lowest_found = (-1);
+	Stat.highest_found = (-1);
 
 	Buf = NewStrBufPlain(HKEY("_TASKS_"));
 	gotoroom(Buf);
@@ -92,21 +99,22 @@ void tasks_section(void) {
 		num_msgs = 0;
 	}
 	else {
-		num_msgs = load_msg_ptrs("MSGS ALL", 0);
+		num_msgs = load_msg_ptrs("MSGS ALL", &Stat);
 	}
 
 	if (num_msgs > 0) {
 		at = GetNewHashPos(WCC->summ, 0);
 		while (GetNextHashPos(WCC->summ, at, &HKLen, &HashKey, &vMsg)) {
 			Msg = (message_summary*) vMsg;		
-			display_task(Msg, 0);
+			tasks_LoadMsgFromServer(NULL, NULL, Msg, 0, 0);
 		}
+		DeleteHashPos(&at);
 	}
 
 	if (calendar_summary_view() < 1) {
-		wprintf("<i>");
-		wprintf(_("(None)"));
-		wprintf("</i><br />\n");
+		wc_printf("<i>");
+		wc_printf(_("(None)"));
+		wc_printf("</i><br />\n");
 	}
 }
 
@@ -115,6 +123,7 @@ void tasks_section(void) {
  * Calendar section
  */
 void calendar_section(void) {
+	char cmd[SIZ];
 	int num_msgs = 0;
 	HashPos *at;
 	const char *HashKey;
@@ -122,9 +131,15 @@ void calendar_section(void) {
 	void *vMsg;
 	message_summary *Msg;
 	wcsession *WCC = WC;
-	struct calview c;
 	StrBuf *Buf;
+	void *v = NULL;
+	SharedMessageStatus Stat;
 
+	memset(&Stat, 0, sizeof(SharedMessageStatus));
+	Stat.maxload = 10000;
+	Stat.lowest_found = (-1);
+	Stat.highest_found = (-1);
+	
 	Buf = NewStrBufPlain(HKEY("_CALENDAR_"));
 	gotoroom(Buf);
 	FreeStrBuf(&Buf);
@@ -132,23 +147,28 @@ void calendar_section(void) {
 		num_msgs = 0;
 	}
 	else {
-		num_msgs = load_msg_ptrs("MSGS ALL", 0);
+		num_msgs = load_msg_ptrs("MSGS ALL", &Stat);
 	}
-
-	parse_calendar_view_request(&c);
+	calendar_GetParamsGetServerCall(&Stat, 
+					&v,
+					readnew, 
+					cmd, 
+					sizeof(cmd));
 
 	if (num_msgs > 0) {
 		at = GetNewHashPos(WCC->summ, 0);
 		while (GetNextHashPos(WCC->summ, at, &HKLen, &HashKey, &vMsg)) {
 			Msg = (message_summary*) vMsg;		
-			load_calendar_item(Msg, 0, &c);
+			calendar_LoadMsgFromServer(NULL, &v, Msg, 0, 0);
 		}
+		DeleteHashPos(&at);
 	}
 	if (calendar_summary_view() < 1) {
-		wprintf("<i>");
-		wprintf(_("(Nothing)"));
-		wprintf("</i><br />\n");
+		wc_printf("<i>");
+		wc_printf(_("(Nothing)"));
+		wc_printf("</i><br />\n");
 	}
+	__calendar_Cleanup(&v);
 }
 
 /*
@@ -177,82 +197,82 @@ void server_info_section(void) {
  * not people I consider worthwhile, I still want them to use WebCit.
  */
 void summary_inner_div(void) {
-	wprintf("<div class=\"fix_scrollbar_bug\">"
+	wc_printf("<div class=\"fix_scrollbar_bug\">"
 		"<table width=\"100%%\" cellspacing=\"10\" cellpadding=\"0\">"
 		"<tr valign=top>");
 
 	/*
 	 * Column One
 	 */
-	wprintf("<td width=33%%>");
-	wprintf("<div class=\"box\">");	
-	wprintf("<div class=\"boxlabel\">");	
-	wprintf(_("Messages"));
-	wprintf("</div><div class=\"boxcontent\">");	
-	wprintf("<div id=\"msg_inner\">");	
+	wc_printf("<td width=33%%>");
+	wc_printf("<div class=\"box\">");	
+	wc_printf("<div class=\"boxlabel\">");	
+	wc_printf(_("Messages"));
+	wc_printf("</div><div class=\"boxcontent\">");	
+	wc_printf("<div id=\"msg_inner\">");	
 	new_messages_section();
-	wprintf("</div></div></div>");
-	wprintf("</td>");
+	wc_printf("</div></div></div>");
+	wc_printf("</td>");
 
 	/*
 	 * Column Two 
 	 */
-	wprintf("<td width=33%%>");
-	wprintf("<div class=\"box\">");	
-	wprintf("<div class=\"boxlabel\">");	
-	wprintf(_("Tasks"));
-	wprintf("</div><div class=\"boxcontent\">");	
-	wprintf("<div id=\"tasks_inner\">");	
+	wc_printf("<td width=33%%>");
+	wc_printf("<div class=\"box\">");	
+	wc_printf("<div class=\"boxlabel\">");	
+	wc_printf(_("Tasks"));
+	wc_printf("</div><div class=\"boxcontent\">");	
+	wc_printf("<div id=\"tasks_inner\">");	
 	tasks_section();
-	wprintf("</div></div></div>");
-	wprintf("</td>");
+	wc_printf("</div></div></div>");
+	wc_printf("</td>");
 
 	/*
 	 * Column Three
 	 */
-	wprintf("<td width=33%%>");
-	wprintf("<div class=\"box\">");	
-	wprintf("<div class=\"boxlabel\">");	
-	wprintf(_("Today&nbsp;on&nbsp;your&nbsp;calendar"));
-	wprintf("</div><div class=\"boxcontent\">");	
-	wprintf("<div id=\"calendar_inner\">");	
+	wc_printf("<td width=33%%>");
+	wc_printf("<div class=\"box\">");	
+	wc_printf("<div class=\"boxlabel\">");	
+	wc_printf(_("Today&nbsp;on&nbsp;your&nbsp;calendar"));
+	wc_printf("</div><div class=\"boxcontent\">");	
+	wc_printf("<div id=\"calendar_inner\">");	
 	calendar_section();
-	wprintf("</div></div></div>");
-	wprintf("</td>");
+	wc_printf("</div></div></div>");
+	wc_printf("</td>");
 
-	wprintf("</tr><tr valign=top>");
+	wc_printf("</tr><tr valign=top>");
 
 	/*
 	 * Row Two - Column One
 	 */
-	wprintf("<td colspan=2>");
-	wprintf("<div class=\"box\">");	
-	wprintf("<div class=\"boxlabel\">");	
-	wprintf(_("Who's&nbsp;online&nbsp;now"));
-	wprintf("</div><div class=\"boxcontent\">");	
-	wprintf("<div id=\"who_inner\">");	
+	wc_printf("<td colspan=2>");
+	wc_printf("<div class=\"box\">");	
+	wc_printf("<div class=\"boxlabel\">");	
+	wc_printf(_("Who's&nbsp;online&nbsp;now"));
+	wc_printf("</div><div class=\"boxcontent\">");	
+	wc_printf("<div id=\"who_inner\">");	
 	do_template("wholistsummarysection", NULL);
-	wprintf("</div></div></div>");
-	wprintf("</td>");
+	wc_printf("</div></div></div>");
+	wc_printf("</td>");
 
 	/*
 	 * Row Two - Column Two
 	 */
-	wprintf("<td width=33%%>");
-	wprintf("<div class=\"box\">");	
-	wprintf("<div class=\"boxlabel\">");	
-	wprintf(_("About&nbsp;this&nbsp;server"));
-	wprintf("</div><div class=\"boxcontent\">");	
-	wprintf("<div id=\"info_inner\">");	
+	wc_printf("<td width=33%%>");
+	wc_printf("<div class=\"box\">");	
+	wc_printf("<div class=\"boxlabel\">");	
+	wc_printf(_("About&nbsp;this&nbsp;server"));
+	wc_printf("</div><div class=\"boxcontent\">");	
+	wc_printf("<div id=\"info_inner\">");	
 	server_info_section();
-	wprintf("</div></div></div>");
-	wprintf("</td>");
+	wc_printf("</div></div></div>");
+	wc_printf("</td>");
 
 
 	/*
 	 * End of columns
 	 */
-	wprintf("</tr></table>");
+	wc_printf("</tr></table>");
 }
 
 
@@ -263,33 +283,33 @@ void summary(void) {
 	char title[256];
 
 	output_headers(1, 1, 2, 0, 0, 0);
-	wprintf("<div id=\"banner\">\n");
-	wprintf("<div class=\"room_banner\">");
-        wprintf("<img src=\"static/summscreen_48x.gif\">");
-        wprintf("<h1>");
+	wc_printf("<div id=\"banner\">\n");
+	wc_printf("<div class=\"room_banner\">");
+        wc_printf("<img src=\"static/summscreen_48x.gif\">");
+        wc_printf("<h1>");
         snprintf(title, sizeof title, _("Summary page for %s"), ChrPtr(WC->wc_fullname));
         escputs(title);
-        wprintf("</h1><h2>");
+        wc_printf("</h1><h2>");
         output_date();
-        wprintf("</h2></div>");
-	wprintf("<div id=\"actiondiv\">");
-	wprintf("<ul class=\"room_actions\">\n");
-	wprintf("<li class=\"start_page\">");
+        wc_printf("</h2></div>");
+	wc_printf("<div id=\"actiondiv\">");
+	wc_printf("<ul class=\"room_actions\">\n");
+	wc_printf("<li class=\"start_page\">");
 	offer_start_page(NULL, &NoCtx);
-        wprintf("</li></ul>");
-        wprintf("</div>");
-        wprintf("</div>");
+        wc_printf("</li></ul>");
+        wc_printf("</div>");
+        wc_printf("</div>");
 
 	/*
 	 * You guessed it ... we're going to refresh using ajax.
 	 * In the future we might consider updating individual sections of the summary
 	 * instead of the whole thing.
 	 */
-	wprintf("<div id=\"content\" class=\"service\">\n");
+	wc_printf("<div id=\"content\" class=\"service\">\n");
 	summary_inner_div();
-	wprintf("</div>\n");
+	wc_printf("</div>\n");
 
-	wprintf(
+	wc_printf(
 		"<script type=\"text/javascript\">					"
 		" new Ajax.PeriodicalUpdater('msg_inner', 'new_messages_html',		"
 		"                            { method: 'get', frequency: 60 }  );	"
@@ -309,11 +329,11 @@ void
 InitModule_SUMMARY
 (void)
 {
-	WebcitAddUrlHandler(HKEY("new_messages_html"), new_messages_section, AJAX);
-	WebcitAddUrlHandler(HKEY("tasks_inner_html"), tasks_section, AJAX);
-	WebcitAddUrlHandler(HKEY("calendar_inner_html"), calendar_section, AJAX);
-	WebcitAddUrlHandler(HKEY("mini_calendar"), ajax_mini_calendar, AJAX);
-	WebcitAddUrlHandler(HKEY("summary"), summary, 0);
-	WebcitAddUrlHandler(HKEY("summary_inner_div"), summary_inner_div, AJAX);
+	WebcitAddUrlHandler(HKEY("new_messages_html"), "", 0, new_messages_section, AJAX);
+	WebcitAddUrlHandler(HKEY("tasks_inner_html"), "", 0, tasks_section, AJAX);
+	WebcitAddUrlHandler(HKEY("calendar_inner_html"), "", 0, calendar_section, AJAX);
+	WebcitAddUrlHandler(HKEY("mini_calendar"), "", 0, ajax_mini_calendar, AJAX);
+	WebcitAddUrlHandler(HKEY("summary"), "", 0, summary, 0);
+	WebcitAddUrlHandler(HKEY("summary_inner_div"), "", 0, summary_inner_div, AJAX);
 }
 
