@@ -1,5 +1,5 @@
 /*
- * $Id: useredit.c 8021 2009-11-08 20:33:04Z dothebart $
+ * $Id: useredit.c 8290 2010-02-09 20:01:49Z dothebart $
  */
 
 #include "webcit.h"
@@ -255,7 +255,7 @@ HashList *iterate_load_userlist(StrBuf *Target, WCTemplputParams *TP)
 {
 	int Done = 0;
 	CompareFunc SortIt;
-	HashList *Hash;
+	HashList *Hash = NULL;
 	StrBuf *Buf;
 	UserListEntry* ul;
 	char nnn[64];
@@ -290,10 +290,9 @@ HashList *iterate_load_userlist(StrBuf *Target, WCTemplputParams *TP)
 			SortByPayload(Hash, SortIt);
 		else 
 			SortByPayload(Hash, CompareUID);
-		return Hash;
         }
 	FreeStrBuf(&Buf);
-	return NULL;
+	return Hash;
 }
 
 
