@@ -1,14 +1,12 @@
 /*
- * $Id: vcard_edit.c 7215 2009-03-13 18:14:44Z ajc $
+ * $Id: vcard_edit.c 7939 2009-10-23 12:51:53Z dothebart $
  */
 
 #include "webcit.h"
+#include "calendar.h"
 
-
-/**
- * \brief Record compare function for sorting address book indices
- * \param ab1 adressbook one
- * \param ab2 adressbook two
+/*
+ * Record compare function for sorting address book indices
  */
 int abcmp(const void *ab1, const void *ab2) {
 	return(strcasecmp(
@@ -18,11 +16,9 @@ int abcmp(const void *ab1, const void *ab2) {
 }
 
 
-/**
- * \brief Helper function for do_addrbook_view()
+/*
+ * Helper function for do_addrbook_view()
  * Converts a name into a three-letter tab label
- * \param tabbuf the tabbuffer to add name to
- * \param name the name to add to the tabbuffer
  */
 void nametab(char *tabbuf, long len, char *name) {
 	stresc(tabbuf, len, name, 0, 0);
@@ -33,45 +29,40 @@ void nametab(char *tabbuf, long len, char *name) {
 }
 
 
-/**
- * \brief display the adressbook overview
- * \param msgnum the citadel message number
- * \param alpha what????
+/*
+ * display the adressbook overview
  */
 void display_addressbook(long msgnum, char alpha) {
 	//char buf[SIZ];
 	/* char mime_partnum[SIZ]; */
-/* 	char mime_filename[SIZ]; */
-/* 	char mime_content_type[SIZ]; */
 	///char mime_disposition[SIZ];
 	//int mime_length;
 	char vcard_partnum[SIZ];
-	char *vcard_source = NULL;
+	StrBuf *vcard_source = NULL;
 	message_summary summ;////TODO: this will leak
 
 	memset(&summ, 0, sizeof(summ));
-	///safestrncpy(summ.subj, _("(no subject)"), sizeof summ.subj);
-///Load Message headers
-//	Msg = 
+	// safestrncpy(summ.subj, _("(no subject)"), sizeof summ.subj);
+	// Load Message headers
 	if (!IsEmptyStr(vcard_partnum)) {
 		vcard_source = load_mimepart(msgnum, vcard_partnum);
 		if (vcard_source != NULL) {
 
-			/** Display the summary line */
+			/* Display the summary line */
 			display_vcard(WC->WBuf, vcard_source, alpha, 0, NULL, msgnum);
 
-			/** If it's my vCard I can edit it */
+			/* If it's my vCard I can edit it */
 			if (	(!strcasecmp(ChrPtr(WC->wc_roomname), USERCONFIGROOM))
 				|| (!strcasecmp(&(ChrPtr(WC->wc_roomname)[11]), USERCONFIGROOM))
 				|| (WC->wc_view == VIEW_ADDRESSBOOK)
 			) {
-				wprintf("<a href=\"edit_vcard?"
+				wc_printf("<a href=\"edit_vcard?"
 					"msgnum=%ld&partnum=%s\">",
 					msgnum, vcard_partnum);
-				wprintf("[%s]</a>", _("edit"));
+				wc_printf("[%s]</a>", _("edit"));
 			}
 
-			free(vcard_source);
+			FreeStrBuf(&vcard_source);
 		}
 	}
 
@@ -79,9 +70,8 @@ void display_addressbook(long msgnum, char alpha) {
 
 
 
-/**
- * \brief  If it's an old "Firstname Lastname" style record, try to convert it.
- * \param namebuf name to analyze, reverse if nescessary
+/*
+ * If it's an old "Firstname Lastname" style record, try to convert it.
  */
 void lastfirst_firstlast(char *namebuf) {
 	char firstname[SIZ];
@@ -100,59 +90,66 @@ void lastfirst_firstlast(char *namebuf) {
 	sprintf(namebuf, "%s; %s", lastname, firstname);
 }
 
-/**
- * \brief fetch what??? name
- * \param msgnum the citadel message number
- * \param namebuf where to put the name in???
+
+
+wc_mime_attachment *load_vcard(message_summary *Msg) 
+{
+	HashPos  *it;
+	StrBuf *FoundCharset = NewStrBuf();
+	StrBuf *Error;
+	void *vMime;
+	const char *Key;
+	long len;
+	wc_mime_attachment *Mime;
+	wc_mime_attachment *VCMime = NULL;
+
+	Msg->MsgBody =  (wc_mime_attachment*) malloc(sizeof(wc_mime_attachment));
+	memset(Msg->MsgBody, 0, sizeof(wc_mime_attachment));
+	Msg->MsgBody->msgnum = Msg->msgnum;
+
+	load_message(Msg, FoundCharset, &Error);
+
+	FreeStrBuf(&FoundCharset);
+	/* look up the vcard... */
+	it = GetNewHashPos(Msg->AllAttach, 0);
+	while (GetNextHashPos(Msg->AllAttach, it, &len, &Key, &vMime) && 
+	       (vMime != NULL)) 
+	{
+		Mime = (wc_mime_attachment*) vMime;
+		if ((strcmp(ChrPtr(Mime->ContentType),
+			   "text/x-vcard") == 0) ||
+		    (strcmp(ChrPtr(Mime->ContentType),
+			    "text/vcard") == 0))
+		{
+			VCMime = Mime;
+			break;
+		}
+	}
+	DeleteHashPos(&it);
+	if (VCMime == NULL)
+		return NULL;
+
+	MimeLoadData(VCMime);
+	return VCMime;
+}
+
+/*
+ * fetch the display name off a vCard
  */
 void fetch_ab_name(message_summary *Msg, char **namebuf) {
-	char buf[SIZ];
-	char mime_partnum[SIZ];
-	char mime_filename[SIZ];
-	char mime_content_type[SIZ];
-	char mime_disposition[SIZ];
-	int mime_length;
-	char vcard_partnum[SIZ];
-	char *vcard_source = NULL;
-	int i, len;
-	message_summary summ;/// TODO this will lak
+	long len;
+	int i;
+	wc_mime_attachment *VCMime = NULL;
 
 	if (namebuf == NULL) return;
 
-	memset(&summ, 0, sizeof(summ));
-	//////safestrncpy(summ.subj, "(no subject)", sizeof summ.subj);
+	VCMime = load_vcard(Msg);
+	if (VCMime == NULL)
+		return;
 
-	sprintf(buf, "MSG0 %ld|0", Msg->msgnum);	/** unfortunately we need the mime info now */
-	serv_puts(buf);
-	serv_getln(buf, sizeof buf);
-	if (buf[0] != '1') return;
+	/* Grab the name off the card */
+	display_vcard(WC->WBuf, VCMime->Data, 0, 0, namebuf, Msg->msgnum);
 
-	while (serv_getln(buf, sizeof buf), strcmp(buf, "000")) {
-		if (!strncasecmp(buf, "part=", 5)) {
-			extract_token(mime_filename, &buf[5], 1, '|', sizeof mime_filename);
-			extract_token(mime_partnum, &buf[5], 2, '|', sizeof mime_partnum);
-			extract_token(mime_disposition, &buf[5], 3, '|', sizeof mime_disposition);
-			extract_token(mime_content_type, &buf[5], 4, '|', sizeof mime_content_type);
-			mime_length = extract_int(&buf[5], 5);
-
-			if (  (!strcasecmp(mime_content_type, "text/x-vcard"))
-			   || (!strcasecmp(mime_content_type, "text/vcard")) ) {
-				strcpy(vcard_partnum, mime_partnum);
-			}
-
-		}
-	}
-
-	if (!IsEmptyStr(vcard_partnum)) {
-		vcard_source = load_mimepart(Msg->msgnum, vcard_partnum);
-		if (vcard_source != NULL) {
-
-			/* Grab the name off the card */
-			display_vcard(WC->WBuf, vcard_source, 0, 0, namebuf, Msg->msgnum);
-
-			free(vcard_source);
-		}
-	}
 	if (*namebuf != NULL) {
 		lastfirst_firstlast(*namebuf);
 		striplt(*namebuf);
@@ -170,9 +167,8 @@ void fetch_ab_name(message_summary *Msg, char **namebuf) {
 
 
 
-/**
- * \brief Turn a vCard "n" (name) field into something displayable.
- * \param name the name field to convert
+/*
+ * Turn a vCard "n" (name) field into something displayable.
  */
 void vcard_n_prettyize(char *name)
 {
@@ -209,14 +205,12 @@ void vcard_n_prettyize(char *name)
 
 
 
-/**
- * \brief preparse a vcard name
+/*
+ * preparse a vcard name
  * display_vcard() calls this after parsing the textual vCard into
  * our 'struct vCard' data object.
  * This gets called instead of display_parsed_vcard() if we are only looking
  * to extract the person's name instead of displaying the card.
- * \param v the vcard to retrieve the name from
- * \param storename where to put the name at
  */
 void fetchname_parsed_vcard(struct vCard *v, char **storename) {
 	char *name;
@@ -259,7 +253,12 @@ void fetchname_parsed_vcard(struct vCard *v, char **storename) {
 				len);
 		}
 		else {
-			*storename = strdup(name);
+			size_t len;
+
+			len = strlen (name);
+			
+			*storename = malloc(len + 3); /* \0 + eventualy missing ', '*/
+			memcpy(*storename, name, len + 1);
 		}
 		/* vcard_n_prettyize(storename); */
 	}
@@ -268,8 +267,8 @@ void fetchname_parsed_vcard(struct vCard *v, char **storename) {
 
 
 
-/**
- * \brief html print a vcard
+/*
+ * html print a vcard
  * display_vcard() calls this after parsing the textual vCard into
  * our 'struct vCard' data object.
  *
@@ -281,9 +280,9 @@ void fetchname_parsed_vcard(struct vCard *v, char **storename) {
  * fields we understand, and then render them in a pretty fashion at the
  * end.  Then we make a second pass, outputting all the fields we don't
  * understand in a simple two-column name/value format.
- * \param v the vCard to display
- * \param full display all items of the vcard?
- * \param msgnum Citadel message pointer
+ * v		the vCard to display
+ * full		display all items of the vcard?
+ * msgnum	Citadel message pointer
  */
 void display_parsed_vcard(StrBuf *Target, struct vCard *v, int full, long msgnum) {
 	int i, j;
@@ -349,7 +348,7 @@ void display_parsed_vcard(StrBuf *Target, struct vCard *v, int full, long msgnum
 			len = strlen(v->prop[i].value);
 			/* if we have some untagged QP, detect it here. */
 			if (!is_qp && (strstr(v->prop[i].value, "=?")!=NULL))
-				utf8ify_rfc822_string(v->prop[i].value);
+				utf8ify_rfc822_string(&v->prop[i].value);
 
 			if (is_qp) {
 				// %ff can become 6 bytes in utf8 
@@ -370,9 +369,9 @@ void display_parsed_vcard(StrBuf *Target, struct vCard *v, int full, long msgnum
 				thisvalue = strdup(v->prop[i].value);
 			}
 	
-			/** Various fields we may encounter ***/
+			/* Various fields we may encounter ***/
 	
-			/** N is name, but only if there's no FN already there */
+			/* N is name, but only if there's no FN already there */
 			if (!strcasecmp(firsttoken, "n")) {
 				if (IsEmptyStr(fullname)) {
 					strcpy(fullname, thisvalue);
@@ -380,17 +379,17 @@ void display_parsed_vcard(StrBuf *Target, struct vCard *v, int full, long msgnum
 				}
 			}
 	
-			/** FN (full name) is a true 'display name' field */
+			/* FN (full name) is a true 'display name' field */
 			else if (!strcasecmp(firsttoken, "fn")) {
 				strcpy(fullname, thisvalue);
 			}
 
-			/** title */
+			/* title */
 			else if (!strcasecmp(firsttoken, "title")) {
 				strcpy(title, thisvalue);
 			}
 	
-			/** organization */
+			/* organization */
 			else if (!strcasecmp(firsttoken, "org")) {
 				strcpy(org, thisvalue);
 			}
@@ -526,20 +525,21 @@ void display_parsed_vcard(StrBuf *Target, struct vCard *v, int full, long msgnum
 
 
 
-/**
- * \brief  Display a textual vCard
+/*
+ * Display a textual vCard
  * (Converts to a vCard object and then calls the actual display function)
  * Set 'full' to nonzero to display the whole card instead of a one-liner.
  * Or, if "storename" is non-NULL, just store the person's name in that
  * buffer instead of displaying the card at all.
- * \param vcard_source the buffer containing the vcard text
- * \param alpha what???
- * \param full should we usse all lines?
- * \param storename where to store???
- * \param msgnum Citadel message pointer
+ *
+ * vcard_source	the buffer containing the vcard text
+ * alpha	Display only if name begins with this letter of the alphabet
+ * full		Display the full vCard (otherwise just the display name)
+ * storename	If not NULL, also store the display name here
+ * msgnum	Citadel message pointer
  */
 void display_vcard(StrBuf *Target, 
-		   const char *vcard_source, 
+		   StrBuf *vcard_source, 
 		   char alpha, 
 		   int full, 
 		   char **storename, 
@@ -551,7 +551,7 @@ void display_vcard(StrBuf *Target,
 	StrBuf *Buf2;
 	char this_alpha = 0;
 
-	v = vcard_load((char*)vcard_source); ///TODO
+	v = VCardLoad(vcard_source);
 
 	if (v == NULL) return;
 
@@ -568,10 +568,11 @@ void display_vcard(StrBuf *Target,
 	if (storename != NULL) {
 		fetchname_parsed_vcard(v, storename);
 	}
-	else if (	(alpha == 0)
-			|| ((isalpha(alpha)) && (tolower(alpha) == tolower(this_alpha)) )
-			|| ((!isalpha(alpha)) && (!isalpha(this_alpha)))
-		) {
+	else if (
+		(alpha == 0)
+		|| ((isalpha(alpha)) && (tolower(alpha) == tolower(this_alpha)))
+		|| ((!isalpha(alpha)) && (!isalpha(this_alpha)))
+	) {
 		display_parsed_vcard(Target, v, full,msgnum);
 	}
 
@@ -580,10 +581,11 @@ void display_vcard(StrBuf *Target,
 
 
 
-/**
- * \brief Render the address book using info we gathered during the scan
- * \param addrbook the addressbook to render
- * \param num_ab the number of the addressbook
+/*
+ * Render the address book using info we gathered during the scan
+ *
+ * addrbook	the addressbook to render
+ * num_ab	the number of the addressbook
  */
 void do_addrbook_view(addrbookent *addrbook, int num_ab) {
 	int i = 0;
@@ -600,9 +602,9 @@ void do_addrbook_view(addrbookent *addrbook, int num_ab) {
 	char **tablabels;
 
 	if (num_ab == 0) {
-		wprintf("<br /><br /><br /><div align=\"center\"><i>");
-		wprintf(_("This address book is empty."));
-		wprintf("</i></div>\n");
+		wc_printf("<br /><br /><br /><div align=\"center\"><i>");
+		wc_printf(_("This address book is empty."));
+		wc_printf("</i></div>\n");
 		return;
 	}
 
@@ -614,9 +616,9 @@ void do_addrbook_view(addrbookent *addrbook, int num_ab) {
 
 	tablabels = malloc(num_pages * sizeof (char *));
 	if (tablabels == NULL) {
-		wprintf("<br /><br /><br /><div align=\"center\"><i>");
-		wprintf(_("An internal error has occurred."));
-		wprintf("</i></div>\n");
+		wc_printf("<br /><br /><br /><div align=\"center\"><i>");
+		wc_printf(_("An internal error has occurred."));
+		wc_printf("</i></div>\n");
 		return;
 	}
 
@@ -638,43 +640,43 @@ void do_addrbook_view(addrbookent *addrbook, int num_ab) {
 		if ((i / NAMESPERPAGE) != page) {	/* New tab */
 			page = (i / NAMESPERPAGE);
 			if (page > 0) {
-				wprintf("</tr></table>\n");
+				wc_printf("</tr></table>\n");
 				end_tab(page-1, num_pages);
 			}
 			begin_tab(page, num_pages);
-			wprintf("<table border=0 cellspacing=0 cellpadding=3 width=100%%>\n");
+			wc_printf("<table border=0 cellspacing=0 cellpadding=3 width=100%%>\n");
 			displayed = 0;
 		}
 
 		if ((displayed % 4) == 0) {
 			if (displayed > 0) {
-				wprintf("</tr>\n");
+				wc_printf("</tr>\n");
 			}
 			bg = 1 - bg;
-			wprintf("<tr bgcolor=\"#%s\">",
+			wc_printf("<tr bgcolor=\"#%s\">",
 				(bg ? "DDDDDD" : "FFFFFF")
 			);
 		}
 	
-		wprintf("<td>");
+		wc_printf("<td>");
 
-		wprintf("<a href=\"readfwd?startmsg=%ld?is_singlecard=1",
+		wc_printf("<a href=\"readfwd?startmsg=%ld?is_singlecard=1",
 			addrbook[i].ab_msgnum);
-		wprintf("?maxmsgs=1?is_summary=0?alpha=%s\">", bstr("alpha"));
+		wc_printf("?maxmsgs=1?is_summary=0?alpha=%s\">", bstr("alpha"));
 		vcard_n_prettyize(addrbook[i].ab_name);
 		escputs(addrbook[i].ab_name);
-		wprintf("</a></td>\n");
+		wc_printf("</a></td>\n");
 		++displayed;
 	}
 
 	/* Placeholders for empty columns at end */
 	if ((num_ab % 4) != 0) {
 		for (i=0; i<(4-(num_ab % 4)); ++i) {
-			wprintf("<td>&nbsp;</td>");
+			wc_printf("<td>&nbsp;</td>");
 		}
 	}
 
-	wprintf("</tr></table>\n");
+	wc_printf("</tr></table>\n");
 	end_tab((num_pages-1), num_pages);
 
 	begin_tab(num_pages, num_pages);
@@ -696,10 +698,13 @@ void do_addrbook_view(addrbookent *addrbook, int num_ab) {
  * and MIME part number to fetch.  Or, specify -1 for the message number
  * to start with a blank card.
  */
-void do_edit_vcard(long msgnum, char *partnum, char *return_to, const char *force_room) {
-	char buf[SIZ];
-	char *serialized_vcard = NULL;
-	size_t total_len = 0;
+void do_edit_vcard(long msgnum, char *partnum, 
+		   message_summary *VCMsg,
+		   wc_mime_attachment *VCAtt,
+		   const char *return_to, 
+		   const char *force_room) {
+	message_summary *Msg = NULL;
+	wc_mime_attachment *VCMime = NULL;
 	struct vCard *v;
 	int i;
 	char *key, *value;
@@ -753,49 +758,36 @@ void do_edit_vcard(long msgnum, char *partnum, char *return_to, const char *forc
 
 	safestrncpy(whatuser, "", sizeof whatuser);
 
-	if (msgnum >= 0) {
-		sprintf(buf, "MSG0 %ld|1", msgnum);
-		serv_puts(buf);
-		serv_getln(buf, sizeof buf);
-		if (buf[0] != '1') {
-			convenience_page("770000", _("Error"), &buf[4]);
-			return;
-		}
-		while (serv_getln(buf, sizeof buf), strcmp(buf, "000")) {
-			if (!strncasecmp(buf, "from=", 5)) {
-				safestrncpy(whatuser, &buf[5], sizeof whatuser);
-			}
-			else if (!strncasecmp(buf, "node=", 5)) {
-				strcat(whatuser, " @ ");
-				strcat(whatuser, &buf[5]);
-			}
-		}
-	
-		sprintf(buf, "DLAT %ld|%s", msgnum, partnum);
-		serv_puts(buf);
-		serv_getln(buf, sizeof buf);
-		if (buf[0] != '6') {
-			convenience_page("770000", "Error", &buf[4]);
-			return;
-		}
-	
-		total_len = atoi(&buf[4]);
-		serialized_vcard = malloc(total_len + 2);
+	if ((msgnum >= 0) || 
+	    ((VCMsg != NULL) && (VCAtt != NULL)))
+	{
+		if ((VCMsg == NULL) && (VCAtt == NULL)) {
 
-		serv_read(serialized_vcard, total_len);
-		serialized_vcard[total_len] = 0;
-	
-		v = vcard_load(serialized_vcard);
-		free(serialized_vcard);
+			Msg = (message_summary *) malloc(sizeof(message_summary));
+			memset(Msg, 0, sizeof(message_summary));
+			Msg->msgnum = msgnum;
+			VCMime = load_vcard(Msg);
+			if (VCMime == NULL) {
+				convenience_page("770000", _("Error"), "");///TODO: important message
+				DestroyMessageSummary(Msg);
+				return;
+			}
+		
+			v = VCardLoad(VCMime->Data);
+		}
+		else {
+			v = VCardLoad(VCAtt->Data);
+		}
 	
 		/* Populate the variables for our form */
 		i = 0;
 		while (key = vcard_get_prop(v, "", 0, i, 1), key != NULL) {
-			value = vcard_get_prop(v, "", 0, i++, 0);
-
 			char prp[256];	/* property name */
 			char prm[256];	/* parameters */
 
+			value = vcard_get_prop(v, "", 0, i++, 0);
+
+
 			extract_token(prp, key, 0, ';', sizeof prp);
 			safestrncpy(prm, key, sizeof prm);
 			remove_token(prm, 0, ';');
@@ -876,25 +868,25 @@ void do_edit_vcard(long msgnum, char *partnum, char *return_to, const char *forc
 		vcard_free(v);
 	}
 
-	/** Display the form */
+	/* Display the form */
 	output_headers(1, 1, 1, 0, 0, 0);
 
 	svput("BOXTITLE", WCS_STRING, _("Edit contact information"));
 	do_template("beginboxx", NULL);
 
-	wprintf("<form method=\"POST\" action=\"submit_vcard\">\n");
-	wprintf("<input type=\"hidden\" name=\"nonce\" value=\"%d\">\n", WC->nonce);
+	wc_printf("<form method=\"POST\" action=\"submit_vcard\">\n");
+	wc_printf("<input type=\"hidden\" name=\"nonce\" value=\"%d\">\n", WC->nonce);
 
 	if (force_room != NULL) {
-		wprintf("<input type=\"hidden\" name=\"force_room\" value=\"");
+		wc_printf("<input type=\"hidden\" name=\"force_room\" value=\"");
 		escputs(force_room);
-		wprintf("\">\n");
+		wc_printf("\">\n");
 	}
 
-	wprintf("<div class=\"fix_scrollbar_bug\">"
+	wc_printf("<div class=\"fix_scrollbar_bug\">"
 		"<table class=\"vcard_edit_background\"><tr><td>\n");
 
-	wprintf("<table border=0><tr>"
+	wc_printf("<table border=0><tr>"
 		"<td>%s</td>"
 		"<td>%s</td>"
 		"<td>%s</td>"
@@ -902,147 +894,147 @@ void do_edit_vcard(long msgnum, char *partnum, char *return_to, const char *forc
 		"<td>%s</td></tr>\n",
 		_("Prefix"), _("First"), _("Middle"), _("Last"), _("Suffix")
 	);
-	wprintf("<tr><td><input type=\"text\" name=\"prefix\" "
+	wc_printf("<tr><td><input type=\"text\" name=\"prefix\" "
 		"value=\"%s\" maxlength=\"5\" size=\"5\"></td>",
 		prefix);
-	wprintf("<td><input type=\"text\" name=\"firstname\" "
+	wc_printf("<td><input type=\"text\" name=\"firstname\" "
 		"value=\"%s\" maxlength=\"29\"></td>",
 		firstname);
-	wprintf("<td><input type=\"text\" name=\"middlename\" "
+	wc_printf("<td><input type=\"text\" name=\"middlename\" "
 		"value=\"%s\" maxlength=\"29\"></td>",
 		middlename);
-	wprintf("<td><input type=\"text\" name=\"lastname\" "
+	wc_printf("<td><input type=\"text\" name=\"lastname\" "
 		"value=\"%s\" maxlength=\"29\"></td>",
 		lastname);
-	wprintf("<td><input type=\"text\" name=\"suffix\" "
+	wc_printf("<td><input type=\"text\" name=\"suffix\" "
 		"value=\"%s\" maxlength=\"10\" size=\"10\"></td></tr></table>\n",
 		suffix);
 
-	wprintf("<table  class=\"vcard_edit_background_alt\">");
-	wprintf("<tr><td>");
+	wc_printf("<table  class=\"vcard_edit_background_alt\">");
+	wc_printf("<tr><td>");
 
-	wprintf(_("Display name:"));
-	wprintf("<br>"
+	wc_printf(_("Display name:"));
+	wc_printf("<br>"
 		"<input type=\"text\" name=\"fullname\" "
 		"value=\"%s\" maxlength=\"40\"><br><br>\n",
 		fullname
 	);
 
-	wprintf(_("Title:"));
-	wprintf("<br>"
+	wc_printf(_("Title:"));
+	wc_printf("<br>"
 		"<input type=\"text\" name=\"title\" "
 		"value=\"%s\" maxlength=\"40\"><br><br>\n",
 		title
 	);
 
-	wprintf(_("Organization:"));
-	wprintf("<br>"
+	wc_printf(_("Organization:"));
+	wc_printf("<br>"
 		"<input type=\"text\" name=\"org\" "
 		"value=\"%s\" maxlength=\"40\"><br><br>\n",
 		org
 	);
 
-	wprintf("</td><td>");
+	wc_printf("</td><td>");
 
-	wprintf("<table border=0>");
-	wprintf("<tr><td>");
-	wprintf(_("PO box:"));
-	wprintf("</td><td>"
+	wc_printf("<table border=0>");
+	wc_printf("<tr><td>");
+	wc_printf(_("PO box:"));
+	wc_printf("</td><td>"
 		"<input type=\"text\" name=\"pobox\" "
 		"value=\"%s\" maxlength=\"29\"></td></tr>\n",
 		pobox);
-	wprintf("<tr><td>");
-	wprintf(_("Address:"));
-	wprintf("</td><td>"
+	wc_printf("<tr><td>");
+	wc_printf(_("Address:"));
+	wc_printf("</td><td>"
 		"<input type=\"text\" name=\"extadr\" "
 		"value=\"%s\" maxlength=\"29\"></td></tr>\n",
 		extadr);
-	wprintf("<tr><td> </td><td>"
+	wc_printf("<tr><td> </td><td>"
 		"<input type=\"text\" name=\"street\" "
 		"value=\"%s\" maxlength=\"29\"></td></tr>\n",
 		street);
-	wprintf("<tr><td>");
-	wprintf(_("City:"));
-	wprintf("</td><td>"
+	wc_printf("<tr><td>");
+	wc_printf(_("City:"));
+	wc_printf("</td><td>"
 		"<input type=\"text\" name=\"city\" "
 		"value=\"%s\" maxlength=\"29\"></td></tr>\n",
 		city);
-	wprintf("<tr><td>");
-	wprintf(_("State:"));
-	wprintf("</td><td>"
+	wc_printf("<tr><td>");
+	wc_printf(_("State:"));
+	wc_printf("</td><td>"
 		"<input type=\"text\" name=\"state\" "
 		"value=\"%s\" maxlength=\"29\"></td></tr>\n",
 		state);
-	wprintf("<tr><td>");
-	wprintf(_("ZIP code:"));
-	wprintf("</td><td>"
+	wc_printf("<tr><td>");
+	wc_printf(_("ZIP code:"));
+	wc_printf("</td><td>"
 		"<input type=\"text\" name=\"zipcode\" "
 		"value=\"%s\" maxlength=\"10\"></td></tr>\n",
 		zipcode);
-	wprintf("<tr><td>");
-	wprintf(_("Country:"));
-	wprintf("</td><td>"
+	wc_printf("<tr><td>");
+	wc_printf(_("Country:"));
+	wc_printf("</td><td>"
 		"<input type=\"text\" name=\"country\" "
 		"value=\"%s\" maxlength=\"29\" width=\"5\"></td></tr>\n",
 		country);
-	wprintf("</table>\n");
+	wc_printf("</table>\n");
 
-	wprintf("</table>\n");
+	wc_printf("</table>\n");
 
-	wprintf("<table border=0><tr><td>");
-	wprintf(_("Home telephone:"));
-	wprintf("</td>"
+	wc_printf("<table border=0><tr><td>");
+	wc_printf(_("Home telephone:"));
+	wc_printf("</td>"
 		"<td><input type=\"text\" name=\"hometel\" "
 		"value=\"%s\" maxlength=\"29\"></td>\n",
 		hometel);
-	wprintf("<td>");
-	wprintf(_("Work telephone:"));
-	wprintf("</td>"
+	wc_printf("<td>");
+	wc_printf(_("Work telephone:"));
+	wc_printf("</td>"
 		"<td><input type=\"text\" name=\"worktel\" "
 		"value=\"%s\" maxlength=\"29\"></td></tr>\n",
 		worktel);
-	wprintf("<tr><td>");
-	wprintf(_("Mobile telephone:"));
-	wprintf("</td>"
+	wc_printf("<tr><td>");
+	wc_printf(_("Mobile telephone:"));
+	wc_printf("</td>"
 		"<td><input type=\"text\" name=\"mobiletel\" "
 		"value=\"%s\" maxlength=\"29\"></td>\n",
 		mobiletel);
-	wprintf("<td>");
-	wprintf(_("Fax number:"));
-	wprintf("</td>"
+	wc_printf("<td>");
+	wc_printf(_("Fax number:"));
+	wc_printf("</td>"
 		"<td><input type=\"text\" name=\"faxtel\" "
 		"value=\"%s\" maxlength=\"29\"></td></tr></table>\n",
 		faxtel);
 
-	wprintf("<table class=\"vcard_edit_background_alt\">");
-	wprintf("<tr><td>");
+	wc_printf("<table class=\"vcard_edit_background_alt\">");
+	wc_printf("<tr><td>");
 
-	wprintf("<table border=0><TR>"
+	wc_printf("<table border=0><TR>"
 		"<td valign=top>");
-	wprintf(_("Primary Internet e-mail address"));
-	wprintf("<br />"
+	wc_printf(_("Primary Internet e-mail address"));
+	wc_printf("<br />"
 		"<input type=\"text\" name=\"primary_inetemail\" "
 		"size=40 maxlength=60 value=\"");
 	escputs(primary_inetemail);
-	wprintf("\"><br />"
+	wc_printf("\"><br />"
 		"</td><td valign=top>");
-	wprintf(_("Internet e-mail aliases"));
-	wprintf("<br />"
+	wc_printf(_("Internet e-mail aliases"));
+	wc_printf("<br />"
 		"<textarea name=\"other_inetemail\" rows=5 cols=40 width=40>");
 	escputs(other_inetemail);
-	wprintf("</textarea></td></tr></table>\n");
+	wc_printf("</textarea></td></tr></table>\n");
 
-	wprintf("</td></tr></table>\n");
+	wc_printf("</td></tr></table>\n");
 
-	wprintf("<input type=\"hidden\" name=\"extrafields\" value=\"");
+	wc_printf("<input type=\"hidden\" name=\"extrafields\" value=\"");
 	escputs(extrafields);
-	wprintf("\">\n");
+	wc_printf("\">\n");
 
-	wprintf("<input type=\"hidden\" name=\"return_to\" value=\"");
-	urlescputs(return_to);
-	wprintf("\">\n");
+	wc_printf("<input type=\"hidden\" name=\"return_to\" value=\"");
+	escputs(return_to);
+	wc_printf("\">\n");
 
-	wprintf("<div class=\"buttons\">\n"
+	wc_printf("<div class=\"buttons\">\n"
 		"<input type=\"submit\" name=\"ok_button\" value=\"%s\">"
 		"&nbsp;"
 		"<input type=\"submit\" name=\"cancel_button\" value=\"%s\">"
@@ -1051,13 +1043,16 @@ void do_edit_vcard(long msgnum, char *partnum, char *return_to, const char *forc
 		_("Cancel")
 	);
 	
-	wprintf("</td></tr></table>\n");
+	wc_printf("</td></tr></table>\n");
 	do_template("endbox", NULL);
 	wDumpContent(1);
+	if (Msg != NULL) {
+		DestroyMessageSummary(Msg);
+	}
 }
 
 
-/**
+/*
  *  commit the edits to the citadel server
  */
 void edit_vcard(void) {
@@ -1066,18 +1061,20 @@ void edit_vcard(void) {
 
 	msgnum = lbstr("msgnum");
 	partnum = bstr("partnum");
-	do_edit_vcard(msgnum, partnum, "", NULL);
+	do_edit_vcard(msgnum, partnum, NULL, NULL, "", NULL);
 }
 
 
 
-/**
+/*
  *  parse edited vcard from the browser
  */
 void submit_vcard(void) {
+	wcsession *WCC = WC;
 	struct vCard *v;
 	char *serialized_vcard;
 	char buf[SIZ];
+	StrBuf *Buf;
 	int i;
 
 	if (!havebstr("ok_button")) { 
@@ -1086,7 +1083,34 @@ void submit_vcard(void) {
 	}
 
 	if (havebstr("force_room")) {
-		gotoroom(sbstr("force_room"));
+		if (gotoroom(sbstr("force_room")) != 200) {
+			StrBufAppendBufPlain(WCC->ImportantMsg,
+					     _("Unable to enter the room to save your message"),
+					     -1, 0);
+			StrBufAppendBufPlain(WCC->ImportantMsg,
+					     HKEY(": "), 0);
+			StrBufAppendBuf(WCC->ImportantMsg, sbstr("force_room"), 0);
+			StrBufAppendBufPlain(WCC->ImportantMsg,
+					     HKEY("; "), 0);
+					       
+			StrBufAppendBufPlain(WCC->ImportantMsg,
+					     _("Aborting."),
+					     -1, 0);
+
+			if (!strcmp(bstr("return_to"), "select_user_to_edit")) {
+				select_user_to_edit(NULL);
+			}
+			else if (!strcmp(bstr("return_to"), "do_welcome")) {
+				do_welcome();
+			}
+			else if (!IsEmptyStr(bstr("return_to"))) {
+				http_redirect(bstr("return_to"));
+			}
+			else {
+				readloop(readnew);
+			}
+			return;
+		}
 	}
 
 	sprintf(buf, "ENT0 1|||4||");
@@ -1097,16 +1121,17 @@ void submit_vcard(void) {
 		return;
 	}
 
-	/** Make a vCard structure out of the data supplied in the form */
-
-	snprintf(buf, sizeof buf, "begin:vcard\r\n%s\r\nend:vcard\r\n",
-		bstr("extrafields")
+	/* Make a vCard structure out of the data supplied in the form */
+	Buf = NewStrBuf();
+	StrBufPrintf(Buf, "begin:vcard\r\n%s\r\nend:vcard\r\n",
+		     bstr("extrafields")
 	);
-	v = vcard_load(buf);	/** Start with the extra fields */
+	v = VCardLoad(Buf);	/* Start with the extra fields */
+	FreeStrBuf(&Buf);
 	if (v == NULL) {
-		safestrncpy(WC->ImportantMessage,
+		safestrncpy(WCC->ImportantMessage,
 			_("An error has occurred."),
-			sizeof WC->ImportantMessage
+			sizeof WCC->ImportantMessage
 		);
 		edit_vcard();
 		return;
@@ -1150,9 +1175,9 @@ void submit_vcard(void) {
 	serialized_vcard = vcard_serialize(v);
 	vcard_free(v);
 	if (serialized_vcard == NULL) {
-		safestrncpy(WC->ImportantMessage,
+		safestrncpy(WCC->ImportantMessage,
 			_("An error has occurred."),
-			sizeof WC->ImportantMessage
+			sizeof WCC->ImportantMessage
 		);
 		edit_vcard();
 		return;
@@ -1165,11 +1190,14 @@ void submit_vcard(void) {
 	free(serialized_vcard);
 
 	if (!strcmp(bstr("return_to"), "select_user_to_edit")) {
-		select_user_to_edit(NULL, NULL);
+		select_user_to_edit(NULL);
 	}
 	else if (!strcmp(bstr("return_to"), "do_welcome")) {
 		do_welcome();
 	}
+	else if (!IsEmptyStr(bstr("return_to"))) {
+		http_redirect(bstr("return_to"));
+	}
 	else {
 		readloop(readnew);
 	}
@@ -1183,16 +1211,16 @@ void submit_vcard(void) {
 void display_vcard_photo_img(void)
 {
 	long msgnum = 0L;
-	char *vcard;
+	StrBuf *vcard;
 	struct vCard *v;
 	char *photosrc;
 	const char *contentType;
 	wcsession *WCC = WC;
 
-	msgnum = StrTol(WCC->UrlFragment2);
+	msgnum = StrBufExtract_long(WCC->Hdr->HR.ReqLine, 0, '/');
 	
 	vcard = load_mimepart(msgnum,"1");
-	v = vcard_load(vcard);
+	v = VCardLoad(vcard);
 	
 	photosrc = vcard_get_prop(v, "PHOTO", 1,0,0);
 	FlushStrBuf(WCC->WBuf);
@@ -1203,7 +1231,8 @@ void display_vcard_photo_img(void)
 		hprintf("HTTP/1.1 500 %s\n","Unable to get photo");
 		output_headers(0, 0, 0, 0, 0, 0);
 		hprintf("Content-Type: text/plain\r\n");
-		wprintf(_("Could Not decode vcard photo\n"));
+		begin_burst();
+		wc_printf(_("Could Not decode vcard photo\n"));
 		end_burst();
 		return;
 	}
@@ -1213,14 +1242,104 @@ void display_vcard_photo_img(void)
 	free(photosrc);
 }
 
+typedef struct _vcardview_struct {
+	long is_singlecard;
+	addrbookent *addrbook;
+	long num_ab;
 
+} vcardview_struct;
+
+int vcard_GetParamsGetServerCall(SharedMessageStatus *Stat, 
+				 void **ViewSpecific, 
+				 long oper, 
+				 char *cmd, 
+				 long len)
+{
+	vcardview_struct *VS;
+
+	VS = (vcardview_struct*) malloc (sizeof(vcardview_struct));
+	memset(VS, 0, sizeof(vcardview_struct));
+	*ViewSpecific = (void*)VS;
+
+	VS->is_singlecard = ibstr("is_singlecard");
+	if (VS->is_singlecard != 1) {
+		if (oper == do_search) {
+			snprintf(cmd, len, "MSGS SEARCH|%s", bstr("query"));
+		}
+		else {
+			strcpy(cmd, "MSGS ALL");
+		}
+		Stat->maxmsgs = 9999999;
+	}
+	return 200;
+}
+
+int vcard_LoadMsgFromServer(SharedMessageStatus *Stat, 
+			    void **ViewSpecific, 
+			    message_summary* Msg, 
+			    int is_new, 
+			    int i)
+{
+	vcardview_struct *VS;
+	char *ab_name;
+
+	VS = (vcardview_struct*) *ViewSpecific;
+
+	ab_name = NULL;
+	fetch_ab_name(Msg, &ab_name);
+	if (ab_name == NULL) 
+		return 0;
+	++VS->num_ab;
+	VS->addrbook = realloc(VS->addrbook,
+			       (sizeof(addrbookent) * VS->num_ab) );
+	safestrncpy(VS->addrbook[VS->num_ab-1].ab_name, ab_name,
+		    sizeof(VS->addrbook[VS->num_ab-1].ab_name));
+	VS->addrbook[VS->num_ab-1].ab_msgnum = Msg->msgnum;
+	free(ab_name);
+	return 0;
+}
+
+
+int vcard_RenderView_or_Tail(SharedMessageStatus *Stat, void **ViewSpecific, long oper)
+{
+	const StrBuf *Mime;
+	vcardview_struct *VS;
+
+	VS = (vcardview_struct*) *ViewSpecific;
+	if (VS->is_singlecard)
+		read_message(WC->WBuf, HKEY("view_message"), lbstr("startmsg"), NULL, &Mime);
+	else
+		do_addrbook_view(VS->addrbook, VS->num_ab);	/* Render the address book */
+	return 0;
+}
+
+int vcard_Cleanup(void **ViewSpecific)
+{
+	vcardview_struct *VS;
+
+	VS = (vcardview_struct*) *ViewSpecific;
+	wDumpContent(1);
+	if ((VS != NULL) && 
+	    (VS->addrbook != NULL))
+		free(VS->addrbook);
+	if (VS != NULL) 
+		free(VS);
+	return 0;
+}
 
 void 
 InitModule_VCARD
 (void)
 {
-	WebcitAddUrlHandler(HKEY("edit_vcard"), edit_vcard, 0);
-	WebcitAddUrlHandler(HKEY("submit_vcard"), submit_vcard, 0);
-	WebcitAddUrlHandler(HKEY("vcardphoto"), display_vcard_photo_img, NEED_URL);
+	RegisterReadLoopHandlerset(
+		VIEW_ADDRESSBOOK,
+		vcard_GetParamsGetServerCall,
+		NULL,
+		vcard_LoadMsgFromServer,
+		vcard_RenderView_or_Tail,
+		vcard_Cleanup);
+	WebcitAddUrlHandler(HKEY("edit_vcard"), "", 0, edit_vcard, 0);
+	WebcitAddUrlHandler(HKEY("submit_vcard"), "", 0, submit_vcard, 0);
+	WebcitAddUrlHandler(HKEY("vcardphoto"), "", 0, display_vcard_photo_img, NEED_URL);
 }
 
